import torch
import os

def to_onnx(model, input_size, out_path, batch_size=1, num_channels=3, device='cpu', input_names=None,output_names=None,verbose=True):
    if output_names is None:
        output_names = ['output']
    if input_names is None:
        input_names = ['input']
    w, h = input_size
    dummy_input = torch.randn(batch_size, num_channels, h, w, device=device)
    model.eval()
    torch.onnx.export(model, dummy_input, out_path, verbose=verbose, input_names=input_names,
                      output_names=output_names)



from torchvision.models import resnet18, resnet34, resnet50, resnet101, resnet152, resnext50_32x4d, resnext101_32x8d, \
    wide_resnet101_2, wide_resnet50_2, alexnet, densenet121, densenet161, densenet169, densenet201, googlenet, \
    mnasnet0_5, mnasnet0_75, mnasnet1_0, mnasnet1_3, mobilenet_v2, inception_v3, shufflenet_v2_x0_5, shufflenet_v2_x1_0, \
    shufflenet_v2_x1_5, shufflenet_v2_x2_0


torchvision_models_dict = dict(
    resnet18=resnet18, resnet34=resnet34, resnet50=resnet50, resnet101=resnet101, resnet152=resnet152,
    resnext50_32x4d=resnext50_32x4d, resnext101_32x8d=resnext101_32x8d, wide_resnet101_2=wide_resnet101_2,
    wide_resnet50_2=wide_resnet50_2, alexnet=alexnet, densenet121=densenet121, densenet161=densenet161,
    densenet169=densenet169, densenet201=densenet201, googlenet=googlenet, mnasnet0_5=mnasnet0_5,
    mnasnet0_75=mnasnet0_75, mnasnet1_0=mnasnet1_0, mnasnet1_3=mnasnet1_3, mobilenet_v2=mobilenet_v2,
    inception_v3=inception_v3, shufflenet_v2_x0_5=shufflenet_v2_x0_5, shufflenet_v2_x1_0=shufflenet_v2_x1_0,
    shufflenet_v2_x1_5=shufflenet_v2_x1_5, shufflenet_v2_x2_0=shufflenet_v2_x2_0
)




class ConfigBase:
    def __init__(self,**kwargs):
        for k,v in kwargs.items():
            setattr(self,k,v)
    @classmethod
    def get_class_config_info_dict(cls):
        if issubclass(cls.__base__, ConfigBase):
            dic=cls.__base__.get_class_config_info_dict()
        else:
            dic = {}
        for k, v in cls.__dict__.items():
            if not k.startswith('_') and k[0].upper() == k[0] and v is not None:
                dic[k] = v
        return dic
    def get_config_info_dict(self):
        dic=self.get_class_config_info_dict()
        for k, v in self.__dict__.items():
            if not k.startswith('_') and k[0].upper() == k[0] and v is not None:
                dic[k] = v
        return dic



import numpy as np
from .predict import Predictor,PredictConfigBase
from .dataset import load_image_files
import wk
import cv2


class TestConfigBase:
    TEST_DIR=None
    TEST_OUTPUT_DIR=None
    PREDICT_CONFIG=None
    SPLIT_BY_PROB=False
    predictor=None
    def __init__(self):
        if not self.TEST_OUTPUT_DIR:
            self.TEST_OUTPUT_DIR=self.TEST_DIR+'_test_out'
        wk.remake(self.TEST_OUTPUT_DIR)
        assert self.PREDICT_CONFIG or self.predictor
    def get_predictor(self):
        return self.predictor or Predictor(self.PREDICT_CONFIG)

def test(cfg):
    assert isinstance(cfg,TestConfigBase)
    fs=load_image_files(cfg.TEST_DIR,recursive=True)
    predictor=cfg.get_predictor()
    outFolder=wk.Folder(cfg.TEST_OUTPUT_DIR)
    res={}
    print('Num images: %s'%(len(fs)))
    for i,f in enumerate(fs):
        img=cv2.imread(f)
        if img is None:
            raise Exception('Failed to load %s'%(f))
        cls,prob=predictor.predict(img)

        if cfg.SPLIT_BY_PROB:
            outFolder.openFolder(cls).openFolder('%.1f'%(prob)).eat(f)
        else:
            outFolder.openFolder(cls).eat(f)
        print(i,f,cls)
        res[f]=[cls,prob]
    return res

if __name__ == '__main__':
    test(TestConfigBase())


import wk
import wpcv
import os,shutil,glob

IMAGE_FIEL_EXT=['.jpg','.jpeg','.png','.bmp']

def get_image_paths(dir):
    fs=[]
    for ext in IMAGE_FIEL_EXT:
        fs+=glob.glob(dir+'/*'+ext)
    return fs
def merge(src_dirs,dst=None,overwrite=True):
    dst=dst or os
    wk.remake(dst)
    for dir in src_dirs:
        for cls in os.listdir(dir):
            cls_dir=os.path.join(dir,cls)
            dst_cls_dir=os.path.join(dst,cls)
            cls_fs=get_image_paths(cls_dir)
            wk.copy_files_to(cls_fs,dst_cls_dir,overwrite=overwrite)
            print(dir,cls)
    print('finished.')

if __name__ == '__main__':
    merge(
        src_dirs=[
            '/home/ars/sda5/data/电容/图片整理/work/Cam1/比较好分',
            '/home/ars/sda5/data/电容/图片整理/work/Cam1/比较好分2'
        ],
        dst='/home/ars/sda5/data/电容/图片整理/work/Cam1/合并'
    )







import os, shutil, glob, random
from PIL import Image
import torch
import math
import  numpy as np
IMG_EXTENSIONS = ('.jpg', '.jpeg', '.png', '.ppm', '.bmp', '.pgm', '.tif', '.tiff', '.webp')
from torch.utils.data import DataLoader

def load_image_files(dir,recursive=False):
    fs=[]
    for ext in IMG_EXTENSIONS:
        if recursive:
            fs += glob.glob(dir + '/**/*' + ext,recursive=True)
        else:
            fs += glob.glob(dir + '/*' + ext)
    return fs
def load_image_folder(path):
    classes = os.listdir(path)
    files = {cls: [] for cls in classes}
    for cls in classes:
        cls_dir = os.path.join(path, cls)
        for ext in IMG_EXTENSIONS:
            files[cls] += glob.glob(cls_dir + '/*' + ext)
    return files

def adaptive_duplicate(class_files,nums=None,ratios=None):
    max_length = max([len(fs) for fs in class_files.values()])
    if not nums:
        if not ratios:
            nums=[max_length]*len(class_files)
        else:
            s=sum(ratios)
            ratios=[x/s for x in ratios]
            cls_file_nums=[]
            for i,cls in enumerate(sorted(class_files.keys())):
                cls_file_nums.append(len(class_files[cls]))
            actual_ratios=[x/sum(cls_file_nums) for x in cls_file_nums]
            needs_satisfied=[x1/x2 for x1,x2 in zip(actual_ratios,ratios)]
            index=np.argmax(needs_satisfied)
            ratios=np.array(ratios)
            ratios*=1/ratios[index]
            nums=[int(cls_file_nums[index]*ratios[i]) for i in range(len(class_files.keys()))]
    new_class_files = {}
    for i,cls in enumerate(sorted(class_files.keys())):
        fs=class_files[cls]
        new_class_files[cls] = fs + random.choices(fs, k=nums[i] - len(fs))
    return new_class_files


class ImageFolder:
    def __init__(self, path, balance_classes=False, transform=None):
        self.class_files = load_image_folder(path)
        if balance_classes:
            self.class_files = adaptive_duplicate(self.class_files,ratios=balance_classes if isinstance(balance_classes,(tuple,list)) else None)
        print('Data Info'.center(200,'*'))
        for k,v in self.class_files.items():
            print(k,len(v))
        self.classes = sorted(list(self.class_files.keys()))
        self.cls2idx = {cls: i for i, cls in enumerate(self.classes)}
        self.data = []
        for cls, fs in self.class_files.items():
            self.data += list(zip(fs, [self.cls2idx[cls]] * len(fs)))
        self.transform = transform

    def __getitem__(self, item):
        img_path, cls_idx = self.data[item]
        img = Image.open(img_path)
        if self.transform:
            img = self.transform(img)
        return img, cls_idx

    def __len__(self):
        return len(self.data)
class Dataset(DataLoader):
    def __init__(self, path=None, balance_classes=False, transform=None, data=None, batch_size=16,
                 shuffle=True,num_workers=4):
        data = data or ImageFolder(path, balance_classes, transform)
        super().__init__(data,batch_size=batch_size,shuffle=shuffle,num_workers=num_workers)
def demo():
    pass




import os, shutil, glob, random
from PIL import Image
import torch
import math
import  numpy as np
IMG_EXTENSIONS = ('.jpg', '.jpeg', '.png', '.ppm', '.bmp', '.pgm', '.tif', '.tiff', '.webp')


def load_image_files(dir,recursive=False):
    fs=[]
    for ext in IMG_EXTENSIONS:
        if recursive:
            fs += glob.glob(dir + '/**/*' + ext,recursive=True)
        else:
            fs += glob.glob(dir + '/*' + ext)
    return fs
def load_image_folder(path):
    classes = os.listdir(path)
    files = {cls: [] for cls in classes}
    for cls in classes:
        cls_dir = os.path.join(path, cls)
        for ext in IMG_EXTENSIONS:
            files[cls] += glob.glob(cls_dir + '/*' + ext)
    return files

def adaptive_duplicate(class_files):
    max_length = max([len(fs) for fs in class_files.values()])
    new_class_files = {}
    for cls, fs in class_files.items():
        new_class_files[cls] = fs + random.choices(fs, k=max_length - len(fs))
        # scale=int(max_length//len(fs))
        # new_class_files[cls] = fs*scale
        # print(cls,scale)
    return new_class_files


class ImageFolder:
    def __init__(self, path, balance_classes=False, transform=None):
        self.class_files = load_image_folder(path)
        if balance_classes:
            self.class_files = adaptive_duplicate(self.class_files)
        print('Data Info'.center(200,'*'))
        for k,v in self.class_files.items():
            print(k,len(v))
        self.classes = sorted(list(self.class_files.keys()))
        self.cls2idx = {cls: i for i, cls in enumerate(self.classes)}
        self.data = []
        for cls, fs in self.class_files.items():
            self.data += list(zip(fs, [self.cls2idx[cls]] * len(fs)))
        self.transform = transform

    def __getitem__(self, item):
        img_path, cls_idx = self.data[item]
        img = Image.open(img_path)
        if self.transform:
            img = self.transform(img)
        return img, cls_idx

    def __len__(self):
        return len(self.data)


class Dataset:
    def __init__(self, path=None, balance_classes=False, transform=None, data=None, batch_size=16, device=None,
                 shuffle=True):
        data = data or ImageFolder(path, balance_classes, transform)
        self.data=data
        self.classes = data.classes
        self.num_classes = len(self.classes)
        self.batch_size = batch_size
        self.shuffle = shuffle
        self.index_list = list(range(len(data)))
        if shuffle:
            random.shuffle(self.index_list)
        self.num_batches =math.ceil( len(self.index_list) / self.batch_size)
        self.current_batch_index = -1
        if not device:
            device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        self.device = device
    def __len__(self):
        return self.num_batches
    def __iter__(self):
        return self

    def __next__(self):
        self.current_batch_index += 1
        if self.current_batch_index == self.num_batches:
            if self.shuffle:
                random.shuffle(self.index_list)
            self.current_batch_index = -1
            raise StopIteration
        start_index=self.current_batch_index * self.batch_size
        end_index=None if self.current_batch_index==self.num_batches-1 else (self.current_batch_index + 1) * self.batch_size
        batch_inds = self.index_list[start_index:end_index]
        batch_img = []
        batch_labels = []
        for ind in batch_inds:
            img, cls_idx = self.data[ind]
            img=torch.unsqueeze(img,0)
            batch_img.append(img)
            batch_labels.append(cls_idx)
        batch_img = torch.cat(batch_img, dim=0)
        batch_img = batch_img.to(self.device)
        batch_labels = torch.tensor(batch_labels).to(self.device)
        return batch_img, batch_labels
def demo():
    pass



from .dataset import Dataset,load_image_files,IMG_EXTENSIONS
from .network import load_model
from .predict import Predictor,PredictConfigBase
from .training import train,TrainValConfigBase,val
from .testing import test,TestConfigBase
from .config import *
from .transforms import *


from wcf import train, TrainValConfigBase, val, t, EasyTransform, models_names
from wk import PointDict
from vtgui.app import make_app, SelectDir, SelectFile, VirtualField
from wcf import models_names
import sys


def make_trainval_config(
        cfg, data_cfg
):
    cfg=PointDict(**cfg)
    data_cfg=PointDict(**data_cfg)
    class Config(TrainValConfigBase):
        MODEL_TYPE = cfg.MODEL_TYPE
        GEN_CLASSES_FILE = cfg.GEN_CLASSES_FILE
        USE_tqdm_TRAIN = cfg.USE_tqdm_TRAIN
        INPUT_SIZE = (cfg.INPUT_W, cfg.INPUT_H)
        BATCH_SIZE = cfg.BATCH_SIZE
        MAX_EPOCHS = cfg.MAX_EPOCHS
        BALANCE_CLASSES = cfg.BALANCE_CLASSES
        BALANCE_CLASSES_VAL = cfg.BALANCE_CLASSES_VAL
        VAL_INTERVAL = cfg.VAL_INTERVAL
        WEIGHTS_SAVE_INTERVAL = cfg.WEIGHTS_SAVE_INTERVAL
        WEIGHTS_INIT = cfg.WEIGHTS_INIT
        TRAIN_DIR = cfg.TRAIN_DIR
        VAL_DIR = cfg.VAL_DIR
        INPUT_W = cfg.INPUT_W
        INPUT_H = cfg.INPUT_H
        USE_PRETRAINED = cfg.USE_PRETRAINED
        val_transform = EasyTransform([
            t.Resize(INPUT_SIZE[::-1]),
            t.SaveToDir(cfg.VISUALIZE_RESULT_DIR),
            t.ToTensor(),
        ])
        train_transform = EasyTransform(list(filter(lambda x:x is not None,[
            t.ColorJitter(brightness=data_cfg.BRIGHTNESS, contrast=data_cfg.CONTRAST, saturation=data_cfg.SATURATION, hue=data_cfg.HUE),
            t.RandomHorizontalFlip() if data_cfg.RandomHorizontalFlip else None,
            t.RandomVerticalFlip() if data_cfg.RandomVerticalFlip else None,
            t.RandomRotate(data_cfg.RandomRotate) if data_cfg.RandomRotate else None ,
            t.RandomShear(data_cfg.RandomShear,data_cfg.RandomShear) if data_cfg.RandomShear else None,
            t.RandomTranslate(data_cfg.RandomTranslate) if data_cfg.RandomTranslate else None,
            t.RandomBlur(p=data_cfg.RandomBlur, radius=1) if data_cfg.RandomBlur else None,
            t.RandomSPNoise(p=data_cfg.RandomSPNoise) if data_cfg.RandomSPNoise else None,
            *val_transform,
        ])))
    for k,v in cfg.items():
        setattr(Config,k,v)
    return Config




models = [
    models_names.resnet10,
    models_names.resnet18,
    models_names.resnet50,
    models_names.shufflenet_v2_x0_5,
    models_names.shufflenet_v2_x1_0,
]

def get_default_base_config(**kwargs):
    base_config = dict(
        MODEL_TYPE='resnet18',
        GEN_CLASSES_FILE=True,
        USE_tqdm_TRAIN=True,
        USE_PRETRAINED=True,
        BATCH_SIZE=64,
        MAX_EPOCHS=200,
        BALANCE_CLASSES=True,
        BALANCE_CLASSES_VAL=True,
        VAL_INTERVAL=1,
        WEIGHTS_SAVE_INTERVAL=1,
        WEIGHTS_INIT='weights/training/model_best.pkl',
        INPUT_W=224,
        INPUT_H=224,
        VISUALIZE_RESULT_DIR='data/visualize',
    )
    base_config.update(**kwargs)
    return base_config

def get_default_data_config(**kwargs):
    data_config = dict(
        BRIGHTNESS=0.1,
        CONTRAST=0.05,
        SATURATION=0.05,
        HUE=0.05,
        RandomHorizontalFlip=False,
        RandomVerticalFlip=False,
        RandomRotate=0,
        RandomShear=0,
        RandomTranslate=0,
        RandomBlur=0.3,
        RandomSPNoise=0.3,
    )
    data_config.update(**kwargs)
    return data_config

def get_base_config(train_dir='/home/ars/sda5/data/projects/烟分类/data/烟分类-train',val_dir='/home/ars/sda5/data/projects/烟分类/data/烟分类-val'):
    base_config = dict(
        MODEL_TYPE=VirtualField(title='模型', description='选择模型', default='resnet18', options=models),
        TRAIN_DIR=SelectDir(train_dir),
        VAL_DIR=SelectDir(val_dir),
        GEN_CLASSES_FILE=VirtualField(default=True, title='生成类别文件'),
        USE_tqdm_TRAIN=True,
        USE_PRETRAINED=True,
        BATCH_SIZE=64,
        MAX_EPOCHS=200,
        BALANCE_CLASSES=True,
        BALANCE_CLASSES_VAL=True,
        VAL_INTERVAL=1,
        WEIGHTS_SAVE_INTERVAL=1,
        WEIGHTS_INIT='weights/training/model_best.pkl',
        INPUT_W=224,
        INPUT_H=224,
        VISUALIZE_RESULT_DIR='data/visualize',
    )
    return base_config

def get_data_config():
    data_config = dict(
        BRIGHTNESS=0.1,
        CONTRAST=0.05,
        SATURATION=0.05,
        HUE=0.05,
        RandomHorizontalFlip=False,
        RandomVerticalFlip=False,
        RandomRotate=0,
        RandomShear=0,
        RandomTranslate=0,
        RandomBlur=0.3,
        RandomSPNoise=0.3,
    )
    return data_config


def training_callback(base_cfg, data_cfg={}):
    base_config = get_default_base_config(**base_cfg)
    data_config = get_default_data_config(**data_cfg)
    Config= make_trainval_config(base_config,data_config)
    cfg=Config()
    train(cfg)

def make_training_app(base_config,data_config,function=training_callback,columns=[4, 4],window_size=(1600,800)):
    app = make_app(function=function, args=(base_config, data_config), columns=columns,window_size=window_size)
    return app

def make_example_training_app(train_dir='',val_dir=''):
    base_config = get_base_config(train_dir=train_dir, val_dir=val_dir)
    data_config = get_data_config()
    return make_training_app(base_config,data_config)

def make_simple_training_app(train_dir='', val_dir='', default_base_config=None, default_data_config=None):
    if default_data_config is None:
        default_data_config = {}
    if default_base_config is None:
        default_base_config = {}
    base_config=dict(
        TRAIN_DIR=SelectDir(default=train_dir,title='训练集路径'),
        VAL_DIR=SelectDir(default=val_dir,title='验证集路径'),
    )
    def train(base_config):
        default_base_config.update(**base_config)
        training_callback(default_base_config,default_data_config)
    return make_app(train,args=(base_config,),columns=2,window_size=(1300,800))

def capacitor_app_demo(train_dir='',val_dir=''):
    default_data_config=dict(
        BRIGHTNESS=0.1,
        CONTRAST=0.05,
        SATURATION=0.05,
        HUE=0.1,
        RandomHorizontalFlip=False,
        RandomVerticalFlip=False,
        RandomRotate=360,
        RandomShear=0,
        RandomTranslate=50,
        RandomBlur=0.3,
        RandomSPNoise=0.3,
    )
    default_base_config=dict(
        MODEL_TYPE='resnet18',
        GEN_CLASSES_FILE=True,
        USE_tqdm_TRAIN=True,
        USE_PRETRAINED=False,
        BATCH_SIZE=64,
        MAX_EPOCHS=200,
        BALANCE_CLASSES=True,
        BALANCE_CLASSES_VAL=True,
        VAL_INTERVAL=1,
        WEIGHTS_SAVE_INTERVAL=1,
        WEIGHTS_INIT=None,
        INPUT_W=360,
        INPUT_H=270,
        LOG_INFO=False,
        VISUALIZE_RESULT_DIR='data/visualize',
    )
    return make_simple_training_app(train_dir=train_dir,val_dir=val_dir,default_base_config=default_base_config,default_data_config=default_data_config)
if __name__ == '__main__':
    # make_simple_training_app(
    #     train_dir='/home/ars/sda5/data/chaoyuan/datasets/classify_datasets/公章/train',
    #     val_dir='/home/ars/sda5/data/chaoyuan/datasets/classify_datasets/公章/train',
    # ).run()
    capacitor_app_demo(
        train_dir='/home/ars/sda5/data/projects/无锡和博电容/data2/color1/face2/标注中-train',
        val_dir='/home/ars/sda5/data/projects/无锡和博电容/data2/color1/face2/标注中-val',
    ).run()




from pytorchcv.model_provider import alexnet, alexnetb, zfnet, zfnetb, vgg11, vgg13, vgg16, vgg19, bn_vgg11, bn_vgg13, \
    bn_vgg16, bn_vgg19, bn_vgg11b, bn_vgg13b, bn_vgg16b, bn_vgg19b, bninception, resnet10, resnet12, resnet14, \
    resnetbc14b, resnet16, resnet18_wd4, resnet18_wd2, resnet18_w3d4, resnet18, resnet26, resnetbc26b, resnet34, \
    resnetbc38b, resnet50, resnet50b, resnet101, resnet101b, resnet152, resnet152b, resnet200, resnet200b, preresnet10, \
    preresnet12, preresnet14, preresnetbc14b, preresnet16, preresnet18_wd4, preresnet18_wd2, preresnet18_w3d4, \
    preresnet18, preresnet26, preresnetbc26b, preresnet34, preresnetbc38b, preresnet50, preresnet50b, preresnet101, \
    preresnet101b, preresnet152, preresnet152b, preresnet200, preresnet200b, preresnet269b, resnext14_16x4d, \
    resnext14_32x2d, resnext14_32x4d, resnext26_16x4d, resnext26_32x2d, resnext26_32x4d, resnext38_32x4d, \
    resnext50_32x4d, resnext101_32x4d, resnext101_64x4d, seresnet10, seresnet12, seresnet14, seresnet16, seresnet18, \
    seresnet26, seresnetbc26b, seresnet34, seresnetbc38b, seresnet50, seresnet50b, seresnet101, seresnet101b, \
    seresnet152, seresnet152b, seresnet200, seresnet200b, sepreresnet10, sepreresnet12, sepreresnet14, sepreresnet16, \
    sepreresnet18, sepreresnet26, sepreresnetbc26b, sepreresnet34, sepreresnetbc38b, sepreresnet50, sepreresnet50b, \
    sepreresnet101, sepreresnet101b, sepreresnet152, sepreresnet152b, sepreresnet200, sepreresnet200b, \
    seresnext50_32x4d, seresnext101_32x4d, seresnext101_64x4d, senet16, senet28, senet40, senet52, senet103, senet154, \
    ibn_resnet50, ibn_resnet101, ibn_resnet152, ibnb_resnet50, ibnb_resnet101, ibnb_resnet152, ibn_resnext50_32x4d, \
    ibn_resnext101_32x4d, ibn_resnext101_64x4d, ibn_densenet121, ibn_densenet161, ibn_densenet169, ibn_densenet201, \
    airnet50_1x64d_r2, airnet50_1x64d_r16, airnet101_1x64d_r2, airnext50_32x4d_r2, airnext101_32x4d_r2, \
    airnext101_32x4d_r16, bam_resnet18, bam_resnet34, bam_resnet50, bam_resnet101, bam_resnet152, cbam_resnet18, \
    cbam_resnet34, cbam_resnet50, cbam_resnet101, cbam_resnet152, resattnet56, resattnet92, resattnet128, resattnet164, \
    resattnet200, resattnet236, resattnet452, sknet50, sknet101, sknet152, diaresnet10, diaresnet12, diaresnet14, \
    diaresnetbc14b, diaresnet16, diaresnet18, diaresnet26, diaresnetbc26b, diaresnet34, diaresnetbc38b, diaresnet50, \
    diaresnet50b, diaresnet101, diaresnet101b, diaresnet152, diaresnet152b, diaresnet200, diaresnet200b, diapreresnet10, \
    diapreresnet12, diapreresnet14, diapreresnetbc14b, diapreresnet16, diapreresnet18, diapreresnet26, \
    diapreresnetbc26b, diapreresnet34, diapreresnetbc38b, diapreresnet50, diapreresnet50b, diapreresnet101, \
    diapreresnet101b, diapreresnet152, diapreresnet152b, diapreresnet200, diapreresnet200b, diapreresnet269b, \
    pyramidnet101_a360, diracnet18v2, diracnet34v2, sharesnet18, sharesnet34, sharesnet50, sharesnet50b, sharesnet101, \
    sharesnet101b, sharesnet152, sharesnet152b, densenet121, densenet161, densenet169, densenet201, condensenet74_c4_g4, \
    condensenet74_c8_g8, sparsenet121, sparsenet161, sparsenet169, sparsenet201, sparsenet264, peleenet, wrn50_2, \
    drnc26, drnc42, drnc58, drnd22, drnd38, drnd54, drnd105, dpn68, dpn68b, dpn98, dpn107, dpn131, darknet_ref, \
    darknet_tiny, darknet19, darknet53, channelnet, revnet38, revnet110, revnet164, irevnet301, bagnet9, bagnet17, \
    bagnet33, dla34, dla46c, dla46xc, dla60, dla60x, dla60xc, dla102, dla102x, dla102x2, dla169, msdnet22, fishnet99, \
    fishnet150, espnetv2_wd2, espnetv2_w1, espnetv2_w5d4, espnetv2_w3d2, espnetv2_w2, hrnet_w18_small_v1, \
    hrnet_w18_small_v2, hrnetv2_w18, hrnetv2_w30, hrnetv2_w32, hrnetv2_w40, hrnetv2_w44, hrnetv2_w48, hrnetv2_w64, \
    vovnet27s, vovnet39, vovnet57, selecsls42, selecsls42b, selecsls60, selecsls60b, selecsls84, hardnet39ds, \
    hardnet68ds, hardnet68, hardnet85, xdensenet121_2, xdensenet161_2, xdensenet169_2, xdensenet201_2, squeezenet_v1_0, \
    squeezenet_v1_1, squeezeresnet_v1_0, squeezeresnet_v1_1, sqnxt23_w1, sqnxt23_w3d2, sqnxt23_w2, sqnxt23v5_w1, \
    sqnxt23v5_w3d2, sqnxt23v5_w2, shufflenet_g1_w1, shufflenet_g2_w1, shufflenet_g3_w1, shufflenet_g4_w1, \
    shufflenet_g8_w1, shufflenet_g1_w3d4, shufflenet_g3_w3d4, shufflenet_g1_wd2, shufflenet_g3_wd2, shufflenet_g1_wd4, \
    shufflenet_g3_wd4, shufflenetv2_wd2, shufflenetv2_w1, shufflenetv2_w3d2, shufflenetv2_w2, shufflenetv2b_wd2, \
    shufflenetv2b_w1, shufflenetv2b_w3d2, shufflenetv2b_w2, menet108_8x1_g3, menet128_8x1_g4, menet160_8x1_g8, \
    menet228_12x1_g3, menet256_12x1_g4, menet348_12x1_g3, menet352_12x1_g8, menet456_24x1_g3, mobilenet_w1, \
    mobilenet_w3d4, mobilenet_wd2, mobilenet_wd4, fdmobilenet_w1, fdmobilenet_w3d4, fdmobilenet_wd2, fdmobilenet_wd4, \
    mobilenetv2_w1, mobilenetv2_w3d4, mobilenetv2_wd2, mobilenetv2_wd4, mobilenetv2b_w1, mobilenetv2b_w3d4, \
    mobilenetv2b_wd2, mobilenetv2b_wd4, mobilenetv3_small_w7d20, mobilenetv3_small_wd2, mobilenetv3_small_w3d4, \
    mobilenetv3_small_w1, mobilenetv3_small_w5d4, mobilenetv3_large_w7d20, mobilenetv3_large_wd2, \
    mobilenetv3_large_w3d4, mobilenetv3_large_w1, mobilenetv3_large_w5d4, igcv3_w1, igcv3_w3d4, igcv3_wd2, igcv3_wd4, \
    ghostnet, mnasnet_b1, mnasnet_a1, mnasnet_small, darts, proxylessnas_cpu, proxylessnas_gpu, proxylessnas_mobile, \
    proxylessnas_mobile14, fbnet_cb, xception, inceptionv3, inceptionv4, inceptionresnetv2, polynet, nasnet_4a1056, \
    nasnet_6a4032, pnasnet5large, spnasnet, efficientnet_b0, efficientnet_b1, efficientnet_b2, efficientnet_b3, \
    efficientnet_b4, efficientnet_b5, efficientnet_b6, efficientnet_b7, efficientnet_b8, efficientnet_b0b, \
    efficientnet_b1b, efficientnet_b2b, efficientnet_b3b, efficientnet_b4b, efficientnet_b5b, efficientnet_b6b, \
    efficientnet_b7b, efficientnet_b0c, efficientnet_b1c, efficientnet_b2c, efficientnet_b3c, efficientnet_b4c, \
    efficientnet_b5c, efficientnet_b6c, efficientnet_b7c, efficientnet_b8c, efficientnet_edge_small_b, \
    efficientnet_edge_medium_b, efficientnet_edge_large_b, mixnet_s, mixnet_m, mixnet_l, nin_cifar10, nin_cifar100, \
    nin_svhn, resnet20_cifar10, resnet20_cifar100, resnet20_svhn, resnet56_cifar10, resnet56_cifar100, resnet56_svhn, \
    resnet110_cifar10, resnet110_cifar100, resnet110_svhn, resnet164bn_cifar10, resnet164bn_cifar100, resnet164bn_svhn, \
    resnet272bn_cifar10, resnet272bn_cifar100, resnet272bn_svhn, resnet542bn_cifar10, resnet542bn_cifar100, \
    resnet542bn_svhn, resnet1001_cifar10, resnet1001_cifar100, resnet1001_svhn, resnet1202_cifar10, resnet1202_cifar100, \
    resnet1202_svhn, preresnet20_cifar10, preresnet20_cifar100, preresnet20_svhn, preresnet56_cifar10, \
    preresnet56_cifar100, preresnet56_svhn, preresnet110_cifar10, preresnet110_cifar100, preresnet110_svhn, \
    preresnet164bn_cifar10, preresnet164bn_cifar100, preresnet164bn_svhn, preresnet272bn_cifar10, \
    preresnet272bn_cifar100, preresnet272bn_svhn, preresnet542bn_cifar10, preresnet542bn_cifar100, preresnet542bn_svhn, \
    preresnet1001_cifar10, preresnet1001_cifar100, preresnet1001_svhn, preresnet1202_cifar10, preresnet1202_cifar100, \
    preresnet1202_svhn, resnext20_16x4d_cifar10, resnext20_16x4d_cifar100, resnext20_16x4d_svhn, \
    resnext20_32x2d_cifar10, resnext20_32x2d_cifar100, resnext20_32x2d_svhn, resnext20_32x4d_cifar10, \
    resnext20_32x4d_cifar100, resnext20_32x4d_svhn, resnext29_32x4d_cifar10, resnext29_32x4d_cifar100, \
    resnext29_32x4d_svhn, resnext29_16x64d_cifar10, resnext29_16x64d_cifar100, resnext29_16x64d_svhn, \
    resnext272_1x64d_cifar10, resnext272_1x64d_cifar100, resnext272_1x64d_svhn, resnext272_2x32d_cifar10, \
    resnext272_2x32d_cifar100, resnext272_2x32d_svhn, seresnet20_cifar10, seresnet20_cifar100, seresnet20_svhn, \
    seresnet56_cifar10, seresnet56_cifar100, seresnet56_svhn, seresnet110_cifar10, seresnet110_cifar100, \
    seresnet110_svhn, seresnet164bn_cifar10, seresnet164bn_cifar100, seresnet164bn_svhn, seresnet272bn_cifar10, \
    seresnet272bn_cifar100, seresnet272bn_svhn, seresnet542bn_cifar10, seresnet542bn_cifar100, seresnet542bn_svhn, \
    seresnet1001_cifar10, seresnet1001_cifar100, seresnet1001_svhn, seresnet1202_cifar10, seresnet1202_cifar100, \
    seresnet1202_svhn, sepreresnet20_cifar10, sepreresnet20_cifar100, sepreresnet20_svhn, sepreresnet56_cifar10, \
    sepreresnet56_cifar100, sepreresnet56_svhn, sepreresnet110_cifar10, sepreresnet110_cifar100, sepreresnet110_svhn, \
    sepreresnet164bn_cifar10, sepreresnet164bn_cifar100, sepreresnet164bn_svhn, sepreresnet272bn_cifar10, \
    sepreresnet272bn_cifar100, sepreresnet272bn_svhn, sepreresnet542bn_cifar10, sepreresnet542bn_cifar100, \
    sepreresnet542bn_svhn, sepreresnet1001_cifar10, sepreresnet1001_cifar100, sepreresnet1001_svhn, \
    sepreresnet1202_cifar10, sepreresnet1202_cifar100, sepreresnet1202_svhn, pyramidnet110_a48_cifar10, \
    pyramidnet110_a48_cifar100, pyramidnet110_a48_svhn, pyramidnet110_a84_cifar10, pyramidnet110_a84_cifar100, \
    pyramidnet110_a84_svhn, pyramidnet110_a270_cifar10, pyramidnet110_a270_cifar100, pyramidnet110_a270_svhn, \
    pyramidnet164_a270_bn_cifar10, pyramidnet164_a270_bn_cifar100, pyramidnet164_a270_bn_svhn, \
    pyramidnet200_a240_bn_cifar10, pyramidnet200_a240_bn_cifar100, pyramidnet200_a240_bn_svhn, \
    pyramidnet236_a220_bn_cifar10, pyramidnet236_a220_bn_cifar100, pyramidnet236_a220_bn_svhn, \
    pyramidnet272_a200_bn_cifar10, pyramidnet272_a200_bn_cifar100, pyramidnet272_a200_bn_svhn, densenet40_k12_cifar10, \
    densenet40_k12_cifar100, densenet40_k12_svhn, densenet40_k12_bc_cifar10, densenet40_k12_bc_cifar100, \
    densenet40_k12_bc_svhn, densenet40_k24_bc_cifar10, densenet40_k24_bc_cifar100, densenet40_k24_bc_svhn, \
    densenet40_k36_bc_cifar10, densenet40_k36_bc_cifar100, densenet40_k36_bc_svhn, densenet100_k12_cifar10, \
    densenet100_k12_cifar100, densenet100_k12_svhn, densenet100_k24_cifar10, densenet100_k24_cifar100, \
    densenet100_k24_svhn, densenet100_k12_bc_cifar10, densenet100_k12_bc_cifar100, densenet100_k12_bc_svhn, \
    densenet190_k40_bc_cifar10, densenet190_k40_bc_cifar100, densenet190_k40_bc_svhn, densenet250_k24_bc_cifar10, \
    densenet250_k24_bc_cifar100, densenet250_k24_bc_svhn, xdensenet40_2_k24_bc_cifar10, xdensenet40_2_k24_bc_cifar100, \
    xdensenet40_2_k24_bc_svhn, xdensenet40_2_k36_bc_cifar10, xdensenet40_2_k36_bc_cifar100, xdensenet40_2_k36_bc_svhn, \
    wrn16_10_cifar10, wrn16_10_cifar100, wrn16_10_svhn, wrn28_10_cifar10, wrn28_10_cifar100, wrn28_10_svhn, \
    wrn40_8_cifar10, wrn40_8_cifar100, wrn40_8_svhn, wrn20_10_1bit_cifar10, wrn20_10_1bit_cifar100, wrn20_10_1bit_svhn, \
    wrn20_10_32bit_cifar10, wrn20_10_32bit_cifar100, wrn20_10_32bit_svhn, ror3_56_cifar10, ror3_56_cifar100, \
    ror3_56_svhn, ror3_110_cifar10, ror3_110_cifar100, ror3_110_svhn, ror3_164_cifar10, ror3_164_cifar100, \
    ror3_164_svhn, rir_cifar10, rir_cifar100, rir_svhn, msdnet22_cifar10, resdropresnet20_cifar10, \
    resdropresnet20_cifar100, resdropresnet20_svhn, shakeshakeresnet20_2x16d_cifar10, shakeshakeresnet20_2x16d_cifar100, \
    shakeshakeresnet20_2x16d_svhn, shakeshakeresnet26_2x32d_cifar10, shakeshakeresnet26_2x32d_cifar100, \
    shakeshakeresnet26_2x32d_svhn, shakedropresnet20_cifar10, shakedropresnet20_cifar100, shakedropresnet20_svhn, \
    fractalnet_cifar10, fractalnet_cifar100, diaresnet20_cifar10, diaresnet20_cifar100, diaresnet20_svhn, \
    diaresnet56_cifar10, diaresnet56_cifar100, diaresnet56_svhn, diaresnet110_cifar10, diaresnet110_cifar100, \
    diaresnet110_svhn, diaresnet164bn_cifar10, diaresnet164bn_cifar100, diaresnet164bn_svhn, diaresnet1001_cifar10, \
    diaresnet1001_cifar100, diaresnet1001_svhn, diaresnet1202_cifar10, diaresnet1202_cifar100, diaresnet1202_svhn, \
    diapreresnet20_cifar10, diapreresnet20_cifar100, diapreresnet20_svhn, diapreresnet56_cifar10, \
    diapreresnet56_cifar100, diapreresnet56_svhn, diapreresnet110_cifar10, diapreresnet110_cifar100, \
    diapreresnet110_svhn, diapreresnet164bn_cifar10, diapreresnet164bn_cifar100, diapreresnet164bn_svhn, \
    diapreresnet1001_cifar10, diapreresnet1001_cifar100, diapreresnet1001_svhn, diapreresnet1202_cifar10, \
    diapreresnet1202_cifar100, diapreresnet1202_svhn, isqrtcovresnet18, isqrtcovresnet34, isqrtcovresnet50, \
    isqrtcovresnet50b, isqrtcovresnet101, isqrtcovresnet101b, resneta50b, resneta101b, resneta152b, resnetd50b, \
    resnetd101b, resnetd152b, fastseresnet101b, octresnet10_ad2, octresnet50b_ad2, resnet10_cub, resnet12_cub, \
    resnet14_cub, resnetbc14b_cub, resnet16_cub, resnet18_cub, resnet26_cub, resnetbc26b_cub, resnet34_cub, \
    resnetbc38b_cub, resnet50_cub, resnet50b_cub, resnet101_cub, resnet101b_cub, resnet152_cub, resnet152b_cub, \
    resnet200_cub, resnet200b_cub, seresnet10_cub, seresnet12_cub, seresnet14_cub, seresnetbc14b_cub, seresnet16_cub, \
    seresnet18_cub, seresnet26_cub, seresnetbc26b_cub, seresnet34_cub, seresnetbc38b_cub, seresnet50_cub, \
    seresnet50b_cub, seresnet101_cub, seresnet101b_cub, seresnet152_cub, seresnet152b_cub, seresnet200_cub, \
    seresnet200b_cub, mobilenet_w1_cub, mobilenet_w3d4_cub, mobilenet_wd2_cub, mobilenet_wd4_cub, fdmobilenet_w1_cub, \
    fdmobilenet_w3d4_cub, fdmobilenet_wd2_cub, fdmobilenet_wd4_cub, proxylessnas_cpu_cub, proxylessnas_gpu_cub, \
    proxylessnas_mobile_cub, proxylessnas_mobile14_cub, ntsnet_cub, fcn8sd_resnetd50b_voc, fcn8sd_resnetd101b_voc, \
    fcn8sd_resnetd50b_coco, fcn8sd_resnetd101b_coco, fcn8sd_resnetd50b_ade20k, fcn8sd_resnetd101b_ade20k, \
    fcn8sd_resnetd50b_cityscapes, fcn8sd_resnetd101b_cityscapes, pspnet_resnetd50b_voc, pspnet_resnetd101b_voc, \
    pspnet_resnetd50b_coco, pspnet_resnetd101b_coco, pspnet_resnetd50b_ade20k, pspnet_resnetd101b_ade20k, \
    pspnet_resnetd50b_cityscapes, pspnet_resnetd101b_cityscapes, deeplabv3_resnetd50b_voc, deeplabv3_resnetd101b_voc, \
    deeplabv3_resnetd152b_voc, deeplabv3_resnetd50b_coco, deeplabv3_resnetd101b_coco, deeplabv3_resnetd152b_coco, \
    deeplabv3_resnetd50b_ade20k, deeplabv3_resnetd101b_ade20k, deeplabv3_resnetd50b_cityscapes, \
    deeplabv3_resnetd101b_cityscapes, icnet_resnetd50b_cityscapes, sinet_cityscapes, alphapose_fastseresnet101b_coco, \
    simplepose_resnet18_coco, simplepose_resnet50b_coco, simplepose_resnet101b_coco, simplepose_resnet152b_coco, \
    simplepose_resneta50b_coco, simplepose_resneta101b_coco, simplepose_resneta152b_coco, \
    simplepose_mobile_resnet18_coco, simplepose_mobile_resnet50b_coco, simplepose_mobile_mobilenet_w1_coco, \
    simplepose_mobile_mobilenetv2b_w1_coco, simplepose_mobile_mobilenetv3_small_w1_coco, \
    simplepose_mobile_mobilenetv3_large_w1_coco, lwopenpose2d_mobilenet_cmupan_coco, lwopenpose3d_mobilenet_cmupan_coco, \
    ibppose_coco, prnet, centernet_resnet18_voc, centernet_resnet18_coco, centernet_resnet50b_voc, \
    centernet_resnet50b_coco, centernet_resnet101b_voc, centernet_resnet101b_coco, lffd20x5s320v2_widerface, \
    lffd25x8s560v1_widerface, voca8flame, superpointnet

from pytorchcv.model_provider import _models as ptcv_models_dict






import numpy as np
from torchvision import transforms
from wpcv.utils.data_aug import img_aug, random_float_generator
from wpcv.utils.ops import pil_ops
import os,shutil
import cv2
from PIL import Image
class SaveToDir:
    def __init__(self,path,max_num=100,remake_dir=True,save_name='%s.jpg'):
        self.path=path
        self.max_num=max_num
        self.count=-1
        self.save_name=save_name
        if os.path.exists(path) and remake_dir:
            shutil.rmtree(path)
        if not os.path.exists(path):
            os.makedirs(path)
    def __call__(self, img):
        self.count+=1
        if self.count>=self.max_num:
            return img
        path=os.path.join(self.path,self.save_name%(self.count))
        img.save(path)
        return img

class EasyTransform(list):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.finalized=False
        self.trans=None
    def ColorJitter(self, brightness=0.1, contrast=0.05, saturation=0.05, hue=0.02):
        T = transforms.ColorJitter(brightness=brightness, contrast=contrast, saturation=contrast, hue=hue)
        self.append(T)
        return T

    def RandomHorizontalFlip(self, **kwargs):
        T = transforms.RandomHorizontalFlip(**kwargs)
        self.append(T)
        return T

    def RandomVerticalFlip(self, **kwargs):
        T = transforms.RandomVerticalFlip(**kwargs)
        self.append(T)
        return T

    def RandomRotate(self, degree):
        T = img_aug.RandomApply(pil_ops.rotate, random_params=dict(degree=random_float_generator(degree)))
        self.append(T)
        return T

    def RandomTranslate(self, offset):
        T = img_aug.RandomApply(pil_ops.translate,
                                random_params=dict(offset=random_float_generator(offset, shape=(2,),
                                                                                 dtype=np.int)))
        self.append(T)
        return T

    def RandomShear(self, degree1, degree2):
        T = img_aug.RandomApply(pil_ops.shear_xy,
                                random_params=dict(degree1=random_float_generator(degree1),
                                                   degree2=random_float_generator(degree2)))
        self.append(T)
        return T

    def RandomBlur(self, radius=1, p=0.3):
        T = img_aug.RandomApply(pil_ops.gaussian_blur, radius=radius, p=p)
        self.append(T)
        return T

    def RandomSPNoise(self, p=0.3):
        T = img_aug.RandomApply(pil_ops.sp_noise, p=p)
        self.append(T)
        return T

    def Resize(self, size):
        T = transforms.Resize(size)
        self.append(T)
        return T

    def RandomApply(self, t, p, **kwargs):
        T = img_aug.RandomApply(t, p=p, **kwargs)
        self.append(T)
        return T

    def ToTensor(self):
        T = transforms.ToTensor()
        self.append(T)
        return T

    def SaveToDir(self,path,max_num=100,remake_dir=True,save_name='%s.jpg'):
        T=SaveToDir(path,max_num,remake_dir,save_name)
        self.append(T)
        return T
    def Compose(self, trans):
        T = transforms.Compose(trans)
        self.append(T)
        return T

    def __call__(self, *args, **kwargs):
        if not  self.finalized:
            self.trans=self._final()
            self.finalized=True
        return self.trans(*args,**kwargs)
    def _final(self):
        return transforms.Compose(self)

t=EasyTransform()


from .dataset import Dataset
import os
import torch
from torch.nn import CrossEntropyLoss
import torch.optim
from torch import optim
import numpy as np
from .config import ConfigBase
from wcf.networks.utils import load_model
import logging
import json
import time

time_stamp = time.strftime('[%m%d]', time.localtime(time.time()))


def load_weights(model, weights_path=None, except_keys=[], strict=False):
    if weights_path:
        if os.path.exists(weights_path):
            try:
                state_dict = torch.load(weights_path)
                remove_keys = []
                for k in state_dict.keys():
                    for key in except_keys:
                        if k.startswith(key):
                            remove_keys.append(k)
                for k in remove_keys:
                    del state_dict[k]
                    print('ignore key:', k)
                model.load_state_dict(state_dict, strict=strict)
                print('Weights loaded from %s' % (weights_path))
            except:
                logging.warning('Cannot load pretrained model %s' % (weights_path))

        if not os.path.exists(weights_path):
            logging.warning('weights path %s does not exists.' % (weights_path))
    return model


class TrainValConfigBase(ConfigBase):
    MODEL_TYPE = 'resnet18'
    TIME_STAMP = time_stamp
    PROJECT_TAG = ''
    EXTRA_TAG = ''
    TAG = None
    NUM_CLASSES = None
    TRAIN_DIR = None
    VAL_DIR = None
    DATA_DIR = None
    INPUT_SIZE = (224, 224)
    BATCH_SIZE = 16
    BATCH_SIZE_TRAIN = None
    BATCH_SIZE_VAL = None
    MAX_EPOCHS = 100
    PATIENCE = 20
    LR_INIT = 1e-3
    LR_END = 1e-5
    USE_PRETRAINED = True
    WEIGHTS_INIT = None
    BALANCE_CLASSES = True
    BALANCE_CLASSES_VAL = False
    criterion = CrossEntropyLoss()
    DEVICE = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
    WEIGHTS_SAVE_INTERVAL = 1
    WEIGHTS_SAVE_DIR = 'weights/training'
    WEIGHTS_SAVE_BEST_NAME = 'model_best_{tag}[epoch={epoch}&acc={val_acc:.4f}].pkl'
    WEIGHTS_SAVE_NAME = 'model{tag}.pkl'
    VAL_INTERVAL = 1
    train_transform = None
    val_transform = None
    USE_tqdm_TRAIN = True
    USE_tqdm_VAL = False
    GEN_CLASSES_FILE = False
    CLASSES_FILE_PATH = 'classes.txt'
    LOG_INFO=True

    def get_TAG(self):
        return '%s%s%s%s' % (
        self.PROJECT_TAG or '', self.TIME_STAMP or '', '[%s]' % (self.MODEL_TYPE), self.EXTRA_TAG or '')

    def get_train_data(self):
        return Dataset(path=self.TRAIN_DIR, balance_classes=self.BALANCE_CLASSES, batch_size=self.BATCH_SIZE_TRAIN,
                        transform=self.train_transform)

    def get_val_data(self):
        return Dataset(path=self.VAL_DIR, balance_classes=self.BALANCE_CLASSES_VAL, batch_size=self.BATCH_SIZE_VAL,
                        transform=self.val_transform, shuffle=False)
    def get_classes(self):
        return sorted(os.listdir(self.TRAIN_DIR))
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.BATCH_SIZE_TRAIN = self.BATCH_SIZE_TRAIN or self.BATCH_SIZE
        self.BATCH_SIZE_VAL = self.BATCH_SIZE_VAL or self.BATCH_SIZE
        if self.TAG is None:
            self.TAG = self.get_TAG()
        self.train_data = self.get_train_data()
        self.val_data = self.get_val_data()
        if self.__class__.NUM_CLASSES is None:
            self.__class__.NUM_CLASSES = len(self.get_classes())
        self.model = self.get_model()
        self.model = load_weights(self.model, weights_path=self.WEIGHTS_INIT)
        self.model.to(self.DEVICE)
        if self.GEN_CLASSES_FILE:
            with open(self.CLASSES_FILE_PATH, 'w') as f:
                f.write('\n'.join(self.get_classes()))
        if self.LOG_INFO:
            self.log_info()
            # pass
    def log_info(self):
        print('CONFIG INFO'.center(200, '*'))
        dic = self.get_config_info_dict()
        max_length = 50
        for k, v in dic.items():
            # k=''
            print('%s\t:\t%s' % (k.rjust(max_length, '_'), v))
    def check_params(self):
        assert self.DATA_DIR or self.TRAIN_DIR

    def get_model(self, num_classes=None):
        num_classes = num_classes or self.NUM_CLASSES
        assert num_classes
        model = load_model(self.MODEL_TYPE, num_classes=num_classes, pretrained=self.USE_PRETRAINED)
        return model


class AccuracyMetric:
    def __init__(self):
        self.label_counts = 0
        self.pred_counts = 0
        self.correct_counts = 0
        self.sample_counts = 0

    def analyze(self):
        def non_zero(t):
            '''to prevent some arrays from being divided by zeros'''
            mask = t == 0
            epsilon = 1e-7
            tmp = np.zeros_like(mask)
            tmp.fill(epsilon)
            t = t + tmp * mask
            return t

        if self.sample_counts == 0:
            raise Exception('No samples for training?')
        else:
            recalls = self.correct_counts / non_zero(self.label_counts)
            precisions = self.correct_counts / non_zero(self.pred_counts)
            accuracy = sum(self.correct_counts) / non_zero(self.sample_counts)
        res = dict(
            recalls=recalls,
            precisions=precisions,
            accuracy=accuracy,
        )
        return res

    def batch_step(self, preds, labels):
        preds, labels = preds.cpu(), labels.cpu()
        batch_size, num_classes = preds.shape
        _, preds = torch.max(preds, 1)
        labels = torch.zeros((batch_size, num_classes)).scatter_(-1, torch.unsqueeze(labels, -1), 1)
        preds = torch.zeros((batch_size, num_classes)).scatter_(-1, torch.unsqueeze(preds, -1), 1)
        l=torch.sum(labels, 0).numpy()
        p=torch.sum(preds, 0).numpy()
        c=torch.sum(labels * preds, 0).numpy()
        s=len(labels)
        self.label_counts += l
        self.pred_counts += p
        self.correct_counts += c
        self.sample_counts += s
        # print(self.label_counts,self.pred_counts,self.correct_counts,self.sample_counts)


class MonitoredList:
    def __init__(self, whats_best=None, name=None):
        self.data = []
        self.best = None
        self.best_idx = None
        self.name = name
        if (not whats_best) and name:
            assert isinstance(name, str)
            if 'loss' in name.lower():
                whats_best = 'min'
            elif ('acc' in name.lower()) or ('accuracy' in name.lower()):
                whats_best = 'max'
        self.whats_best = whats_best

    def better(self, a, b):
        assert self.whats_best in ['max', 'min']
        if self.whats_best == 'max':
            return a > b
        else:
            assert self.whats_best == 'min'
            return a < b

    def push(self, data):
        self.data.append(data)
        if len(self.data) == 1:
            self.best = data
            self.best_idx = 0
        elif self.whats_best and self.better(data, self.best):
            self.best = data
            self.best_idx = len(self.data) - 1

    def last_is_best(self):
        assert len(self.data)
        assert self.whats_best
        return self.best_idx == len(self.data) - 1

    def mean(self):
        return np.array(self.data).mean()

    def max(self):
        return np.array(self.data).min(axis=0)

    def min(self):
        return np.array(self.data).max(axis=0)


class HistoryMonitor:
    def __init__(self):
        self.data = dict()

    def push(self, data):
        for k, v in data.items():
            if k not in self.data.keys():
                self.data[k] = MonitoredList(name=k)
            self.data[k].push(v)

    def get(self, k):
        return self.data[k]


def val(cfg):
    assert isinstance(cfg, TrainValConfigBase)
    model = cfg.model
    model.eval()
    losses = []
    val_acc_metric = AccuracyMetric()
    val_data = cfg.val_data
    if cfg.USE_tqdm_VAL:
        import tqdm
        val_data = tqdm.tqdm(val_data)
    for step, (inputs, labels) in enumerate(val_data):
        inputs = inputs.to(cfg.DEVICE)
        labels = labels.to(cfg.DEVICE)
        outputs = model(inputs)
        loss = cfg.criterion(outputs, labels)
        losses.append(loss.item())
        val_acc_metric.batch_step(outputs, labels)
    res = val_acc_metric.analyze()
    res['loss'] = np.mean(losses)
    return res


def train(cfg):
    assert isinstance(cfg, TrainValConfigBase)
    train_data = cfg.train_data
    model = cfg.model
    optimizer = torch.optim.Adam(model.parameters(), lr=cfg.LR_INIT)
    # optimizer = optim.SGD(model.parameters(), lr=cfg.LR_INIT, momentum=0.9)
    lr_scheduler = torch.optim.lr_scheduler.CosineAnnealingLR(optimizer, cfg.MAX_EPOCHS, cfg.LR_END)

    global_step = -1
    saving_interval = int(cfg.WEIGHTS_SAVE_INTERVAL * len(train_data))
    val_interval = int(cfg.VAL_INTERVAL * len(train_data))
    if not os.path.exists(cfg.WEIGHTS_SAVE_DIR):
        os.makedirs(cfg.WEIGHTS_SAVE_DIR)
    train_history = HistoryMonitor()
    val_history = HistoryMonitor()
    for epoch in range(cfg.MAX_EPOCHS):
        model.train()
        losses = []
        train_acc_metric = AccuracyMetric()
        if cfg.USE_tqdm_TRAIN:
            import tqdm
            train_data = tqdm.tqdm(train_data)
        log = '\n' + ('Epoch %s' % (epoch)).center(200, '*')
        print(log)
        for step, (inputs, labels) in enumerate(train_data):
            inputs=inputs.to(cfg.DEVICE)
            labels=labels.to(cfg.DEVICE)
            global_step += 1
            optimizer.zero_grad()
            outputs = model(inputs)
            loss = cfg.criterion(outputs, labels)
            loss.backward()
            losses.append(loss.item())
            optimizer.step()
            train_acc_metric.batch_step(outputs, labels)
            if global_step % saving_interval == 0 and global_step != 0:
                torch.save(model.state_dict(), cfg.WEIGHTS_SAVE_DIR + '/model.pkl')
                context = dict(
                    tag=cfg.TAG,
                    epoch=epoch,
                    step=global_step,
                )
                torch.save(model.state_dict(),
                           os.path.join(cfg.WEIGHTS_SAVE_DIR, cfg.WEIGHTS_SAVE_NAME.format(**context)))
            if global_step % val_interval == 0 and global_step != 0:
                val_res = val(cfg)
                val_history.push(val_res)
                log = '''\nStep:{global_step}\tValLoss:{val_loss:.4f}\tValAccuracy:{val_acc:.4f}\tValRecalls:{val_recalls}\tValPrecisions:{val_precisions}'''.format(
                    epoch=epoch, global_step=global_step, val_loss=val_res['loss'], val_acc=val_res['accuracy'],
                    val_recalls=val_res['recalls'], val_precisions=val_res['precisions']
                )
                print(log)
                context = dict(
                    tag=cfg.TAG,
                    epoch=epoch,
                    step=global_step,
                    val_acc=val_res['accuracy'],
                    val_loss=val_res['loss'],
                )

                if val_history.get('accuracy').last_is_best():
                    torch.save(model.state_dict(),
                               os.path.join(cfg.WEIGHTS_SAVE_DIR, cfg.WEIGHTS_SAVE_BEST_NAME.format(**context)))
                    torch.save(model.state_dict(), cfg.WEIGHTS_SAVE_DIR + f'''/model_best.pkl''')
                    print('New best accuracy: %.4f, model saved.' % (val_res['accuracy']))
                model.train()
        lr_scheduler.step()
        train_res = train_acc_metric.analyze()
        train_res['loss'] = np.mean(losses)
        train_history.push(train_res)
        lr = optimizer.state_dict()['param_groups'][0]['lr']
        log = '''\nTrainLoss:{train_loss:.4f}\tTrainAccuracy:{train_acc:.4f}\tTrainRecalls:{train_recalls}\tTrainPrecisions:{train_precisions}\tLearningRate:{lr:.6f}'''.format(
            epoch=epoch, lr=lr, train_loss=train_res['loss'], train_acc=train_res['accuracy'],
            train_recalls=train_res['recalls'], train_precisions=train_res['precisions']
        )
        print(log)
        torch.save(model.state_dict(), os.path.join(cfg.WEIGHTS_SAVE_DIR, 'model.pkl'))



import torch
import numpy as np
import os,shutil,glob
from PIL import Image
import cv2
from torchvision import transforms
import logging
from .network import load_model


class PredictConfigBase:
    MODEL_TYPE='resnet18'
    CLASSES_PATH=None
    CLASSES=None
    INPUT_SIZE=(224,224)
    transform= None
    DEVICE=torch.device('cuda' if torch.cuda.is_available() else 'cpu')
    WEIGHTS_PATH='weights/model.pkl'
    def __init__(self,**kwargs):
        for k,v in kwargs.items():
            setattr(self,k,v)
        if self.CLASSES_PATH :
            if os.path.exists(self.CLASSES_PATH):
                with open(self.CLASSES_PATH,'r') as f:
                    self.CLASSES=f.read().strip().split('\n')
            else:
                logging.warning('CLASSES_PATH is given but does not exist.')
        assert self.CLASSES
        assert self.WEIGHTS_PATH and os.path.exists(self.WEIGHTS_PATH)
        self.model=self.get_model()
        self.model.load_state_dict(torch.load(self.WEIGHTS_PATH))
        self.model.to(self.DEVICE)
        self.model.eval()
        self.transform=self.transform or self.get_transform()
    def get_transform(self):
        return transforms.Compose([
            transforms.Resize(self.INPUT_SIZE[::-1]),
            transforms.ToTensor(),
        ])
    def get_model(self):
        return load_model(self.MODEL_TYPE,num_classes=len(self.CLASSES))



class Predictor:
    def __init__(self, cfg):
        assert isinstance(cfg,PredictConfigBase)
        self.device=cfg.DEVICE
        self.model=cfg.model
        self.transform=cfg.transform
        self.classes=cfg.CLASSES

    def predict(self,img):
        if isinstance(img,str):
            img=cv2.imread(img)
            img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
            img=Image.fromarray(img)
        elif not isinstance(img,Image.Image):
            img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
            img = Image.fromarray(img)
        im=img
        im = self.transform(im).float()
        im = torch.tensor(im).unsqueeze(0)
        im = im.to(self.device)
        y = self.model(im)
        y=torch.softmax(y,dim=1)
        prob,pred=torch.max(y,dim=1)
        y = int(pred)
        prob=float(prob)
        return self.classes[y] ,prob





from wcf.networks.utils import ptcv_models_dict,torchvision_models_dict

models_dict={}
models_dict.update(ptcv_models_dict)
models_dict.update(torchvision_models_dict)


class models_names:
    alexnet = "alexnet"
    alexnetb = "alexnetb"
    zfnet = "zfnet"
    zfnetb = "zfnetb"
    vgg11 = "vgg11"
    vgg13 = "vgg13"
    vgg16 = "vgg16"
    vgg19 = "vgg19"
    bn_vgg11 = "bn_vgg11"
    bn_vgg13 = "bn_vgg13"
    bn_vgg16 = "bn_vgg16"
    bn_vgg19 = "bn_vgg19"
    bn_vgg11b = "bn_vgg11b"
    bn_vgg13b = "bn_vgg13b"
    bn_vgg16b = "bn_vgg16b"
    bn_vgg19b = "bn_vgg19b"
    bninception = "bninception"
    resnet10 = "resnet10"
    resnet12 = "resnet12"
    resnet14 = "resnet14"
    resnetbc14b = "resnetbc14b"
    resnet16 = "resnet16"
    resnet18_wd4 = "resnet18_wd4"
    resnet18_wd2 = "resnet18_wd2"
    resnet18_w3d4 = "resnet18_w3d4"
    resnet18 = "resnet18"
    resnet26 = "resnet26"
    resnetbc26b = "resnetbc26b"
    resnet34 = "resnet34"
    resnetbc38b = "resnetbc38b"
    resnet50 = "resnet50"
    resnet50b = "resnet50b"
    resnet101 = "resnet101"
    resnet101b = "resnet101b"
    resnet152 = "resnet152"
    resnet152b = "resnet152b"
    resnet200 = "resnet200"
    resnet200b = "resnet200b"
    preresnet10 = "preresnet10"
    preresnet12 = "preresnet12"
    preresnet14 = "preresnet14"
    preresnetbc14b = "preresnetbc14b"
    preresnet16 = "preresnet16"
    preresnet18_wd4 = "preresnet18_wd4"
    preresnet18_wd2 = "preresnet18_wd2"
    preresnet18_w3d4 = "preresnet18_w3d4"
    preresnet18 = "preresnet18"
    preresnet26 = "preresnet26"
    preresnetbc26b = "preresnetbc26b"
    preresnet34 = "preresnet34"
    preresnetbc38b = "preresnetbc38b"
    preresnet50 = "preresnet50"
    preresnet50b = "preresnet50b"
    preresnet101 = "preresnet101"
    preresnet101b = "preresnet101b"
    preresnet152 = "preresnet152"
    preresnet152b = "preresnet152b"
    preresnet200 = "preresnet200"
    preresnet200b = "preresnet200b"
    preresnet269b = "preresnet269b"
    resnext14_16x4d = "resnext14_16x4d"
    resnext14_32x2d = "resnext14_32x2d"
    resnext14_32x4d = "resnext14_32x4d"
    resnext26_16x4d = "resnext26_16x4d"
    resnext26_32x2d = "resnext26_32x2d"
    resnext26_32x4d = "resnext26_32x4d"
    resnext38_32x4d = "resnext38_32x4d"
    resnext50_32x4d = "resnext50_32x4d"
    resnext101_32x4d = "resnext101_32x4d"
    resnext101_64x4d = "resnext101_64x4d"
    seresnet10 = "seresnet10"
    seresnet12 = "seresnet12"
    seresnet14 = "seresnet14"
    seresnet16 = "seresnet16"
    seresnet18 = "seresnet18"
    seresnet26 = "seresnet26"
    seresnetbc26b = "seresnetbc26b"
    seresnet34 = "seresnet34"
    seresnetbc38b = "seresnetbc38b"
    seresnet50 = "seresnet50"
    seresnet50b = "seresnet50b"
    seresnet101 = "seresnet101"
    seresnet101b = "seresnet101b"
    seresnet152 = "seresnet152"
    seresnet152b = "seresnet152b"
    seresnet200 = "seresnet200"
    seresnet200b = "seresnet200b"
    sepreresnet10 = "sepreresnet10"
    sepreresnet12 = "sepreresnet12"
    sepreresnet14 = "sepreresnet14"
    sepreresnet16 = "sepreresnet16"
    sepreresnet18 = "sepreresnet18"
    sepreresnet26 = "sepreresnet26"
    sepreresnetbc26b = "sepreresnetbc26b"
    sepreresnet34 = "sepreresnet34"
    sepreresnetbc38b = "sepreresnetbc38b"
    sepreresnet50 = "sepreresnet50"
    sepreresnet50b = "sepreresnet50b"
    sepreresnet101 = "sepreresnet101"
    sepreresnet101b = "sepreresnet101b"
    sepreresnet152 = "sepreresnet152"
    sepreresnet152b = "sepreresnet152b"
    sepreresnet200 = "sepreresnet200"
    sepreresnet200b = "sepreresnet200b"
    seresnext50_32x4d = "seresnext50_32x4d"
    seresnext101_32x4d = "seresnext101_32x4d"
    seresnext101_64x4d = "seresnext101_64x4d"
    senet16 = "senet16"
    senet28 = "senet28"
    senet40 = "senet40"
    senet52 = "senet52"
    senet103 = "senet103"
    senet154 = "senet154"
    ibn_resnet50 = "ibn_resnet50"
    ibn_resnet101 = "ibn_resnet101"
    ibn_resnet152 = "ibn_resnet152"
    ibnb_resnet50 = "ibnb_resnet50"
    ibnb_resnet101 = "ibnb_resnet101"
    ibnb_resnet152 = "ibnb_resnet152"
    ibn_resnext50_32x4d = "ibn_resnext50_32x4d"
    ibn_resnext101_32x4d = "ibn_resnext101_32x4d"
    ibn_resnext101_64x4d = "ibn_resnext101_64x4d"
    ibn_densenet121 = "ibn_densenet121"
    ibn_densenet161 = "ibn_densenet161"
    ibn_densenet169 = "ibn_densenet169"
    ibn_densenet201 = "ibn_densenet201"
    airnet50_1x64d_r2 = "airnet50_1x64d_r2"
    airnet50_1x64d_r16 = "airnet50_1x64d_r16"
    airnet101_1x64d_r2 = "airnet101_1x64d_r2"
    airnext50_32x4d_r2 = "airnext50_32x4d_r2"
    airnext101_32x4d_r2 = "airnext101_32x4d_r2"
    airnext101_32x4d_r16 = "airnext101_32x4d_r16"
    bam_resnet18 = "bam_resnet18"
    bam_resnet34 = "bam_resnet34"
    bam_resnet50 = "bam_resnet50"
    bam_resnet101 = "bam_resnet101"
    bam_resnet152 = "bam_resnet152"
    cbam_resnet18 = "cbam_resnet18"
    cbam_resnet34 = "cbam_resnet34"
    cbam_resnet50 = "cbam_resnet50"
    cbam_resnet101 = "cbam_resnet101"
    cbam_resnet152 = "cbam_resnet152"
    resattnet56 = "resattnet56"
    resattnet92 = "resattnet92"
    resattnet128 = "resattnet128"
    resattnet164 = "resattnet164"
    resattnet200 = "resattnet200"
    resattnet236 = "resattnet236"
    resattnet452 = "resattnet452"
    sknet50 = "sknet50"
    sknet101 = "sknet101"
    sknet152 = "sknet152"
    diaresnet10 = "diaresnet10"
    diaresnet12 = "diaresnet12"
    diaresnet14 = "diaresnet14"
    diaresnetbc14b = "diaresnetbc14b"
    diaresnet16 = "diaresnet16"
    diaresnet18 = "diaresnet18"
    diaresnet26 = "diaresnet26"
    diaresnetbc26b = "diaresnetbc26b"
    diaresnet34 = "diaresnet34"
    diaresnetbc38b = "diaresnetbc38b"
    diaresnet50 = "diaresnet50"
    diaresnet50b = "diaresnet50b"
    diaresnet101 = "diaresnet101"
    diaresnet101b = "diaresnet101b"
    diaresnet152 = "diaresnet152"
    diaresnet152b = "diaresnet152b"
    diaresnet200 = "diaresnet200"
    diaresnet200b = "diaresnet200b"
    diapreresnet10 = "diapreresnet10"
    diapreresnet12 = "diapreresnet12"
    diapreresnet14 = "diapreresnet14"
    diapreresnetbc14b = "diapreresnetbc14b"
    diapreresnet16 = "diapreresnet16"
    diapreresnet18 = "diapreresnet18"
    diapreresnet26 = "diapreresnet26"
    diapreresnetbc26b = "diapreresnetbc26b"
    diapreresnet34 = "diapreresnet34"
    diapreresnetbc38b = "diapreresnetbc38b"
    diapreresnet50 = "diapreresnet50"
    diapreresnet50b = "diapreresnet50b"
    diapreresnet101 = "diapreresnet101"
    diapreresnet101b = "diapreresnet101b"
    diapreresnet152 = "diapreresnet152"
    diapreresnet152b = "diapreresnet152b"
    diapreresnet200 = "diapreresnet200"
    diapreresnet200b = "diapreresnet200b"
    diapreresnet269b = "diapreresnet269b"
    pyramidnet101_a360 = "pyramidnet101_a360"
    diracnet18v2 = "diracnet18v2"
    diracnet34v2 = "diracnet34v2"
    sharesnet18 = "sharesnet18"
    sharesnet34 = "sharesnet34"
    sharesnet50 = "sharesnet50"
    sharesnet50b = "sharesnet50b"
    sharesnet101 = "sharesnet101"
    sharesnet101b = "sharesnet101b"
    sharesnet152 = "sharesnet152"
    sharesnet152b = "sharesnet152b"
    densenet121 = "densenet121"
    densenet161 = "densenet161"
    densenet169 = "densenet169"
    densenet201 = "densenet201"
    condensenet74_c4_g4 = "condensenet74_c4_g4"
    condensenet74_c8_g8 = "condensenet74_c8_g8"
    sparsenet121 = "sparsenet121"
    sparsenet161 = "sparsenet161"
    sparsenet169 = "sparsenet169"
    sparsenet201 = "sparsenet201"
    sparsenet264 = "sparsenet264"
    peleenet = "peleenet"
    wrn50_2 = "wrn50_2"
    drnc26 = "drnc26"
    drnc42 = "drnc42"
    drnc58 = "drnc58"
    drnd22 = "drnd22"
    drnd38 = "drnd38"
    drnd54 = "drnd54"
    drnd105 = "drnd105"
    dpn68 = "dpn68"
    dpn68b = "dpn68b"
    dpn98 = "dpn98"
    dpn107 = "dpn107"
    dpn131 = "dpn131"
    darknet_ref = "darknet_ref"
    darknet_tiny = "darknet_tiny"
    darknet19 = "darknet19"
    darknet53 = "darknet53"
    channelnet = "channelnet"
    revnet38 = "revnet38"
    revnet110 = "revnet110"
    revnet164 = "revnet164"
    irevnet301 = "irevnet301"
    bagnet9 = "bagnet9"
    bagnet17 = "bagnet17"
    bagnet33 = "bagnet33"
    dla34 = "dla34"
    dla46c = "dla46c"
    dla46xc = "dla46xc"
    dla60 = "dla60"
    dla60x = "dla60x"
    dla60xc = "dla60xc"
    dla102 = "dla102"
    dla102x = "dla102x"
    dla102x2 = "dla102x2"
    dla169 = "dla169"
    msdnet22 = "msdnet22"
    fishnet99 = "fishnet99"
    fishnet150 = "fishnet150"
    espnetv2_wd2 = "espnetv2_wd2"
    espnetv2_w1 = "espnetv2_w1"
    espnetv2_w5d4 = "espnetv2_w5d4"
    espnetv2_w3d2 = "espnetv2_w3d2"
    espnetv2_w2 = "espnetv2_w2"
    hrnet_w18_small_v1 = "hrnet_w18_small_v1"
    hrnet_w18_small_v2 = "hrnet_w18_small_v2"
    hrnetv2_w18 = "hrnetv2_w18"
    hrnetv2_w30 = "hrnetv2_w30"
    hrnetv2_w32 = "hrnetv2_w32"
    hrnetv2_w40 = "hrnetv2_w40"
    hrnetv2_w44 = "hrnetv2_w44"
    hrnetv2_w48 = "hrnetv2_w48"
    hrnetv2_w64 = "hrnetv2_w64"
    vovnet27s = "vovnet27s"
    vovnet39 = "vovnet39"
    vovnet57 = "vovnet57"
    selecsls42 = "selecsls42"
    selecsls42b = "selecsls42b"
    selecsls60 = "selecsls60"
    selecsls60b = "selecsls60b"
    selecsls84 = "selecsls84"
    hardnet39ds = "hardnet39ds"
    hardnet68ds = "hardnet68ds"
    hardnet68 = "hardnet68"
    hardnet85 = "hardnet85"
    xdensenet121_2 = "xdensenet121_2"
    xdensenet161_2 = "xdensenet161_2"
    xdensenet169_2 = "xdensenet169_2"
    xdensenet201_2 = "xdensenet201_2"
    squeezenet_v1_0 = "squeezenet_v1_0"
    squeezenet_v1_1 = "squeezenet_v1_1"
    squeezeresnet_v1_0 = "squeezeresnet_v1_0"
    squeezeresnet_v1_1 = "squeezeresnet_v1_1"
    sqnxt23_w1 = "sqnxt23_w1"
    sqnxt23_w3d2 = "sqnxt23_w3d2"
    sqnxt23_w2 = "sqnxt23_w2"
    sqnxt23v5_w1 = "sqnxt23v5_w1"
    sqnxt23v5_w3d2 = "sqnxt23v5_w3d2"
    sqnxt23v5_w2 = "sqnxt23v5_w2"
    shufflenet_g1_w1 = "shufflenet_g1_w1"
    shufflenet_g2_w1 = "shufflenet_g2_w1"
    shufflenet_g3_w1 = "shufflenet_g3_w1"
    shufflenet_g4_w1 = "shufflenet_g4_w1"
    shufflenet_g8_w1 = "shufflenet_g8_w1"
    shufflenet_g1_w3d4 = "shufflenet_g1_w3d4"
    shufflenet_g3_w3d4 = "shufflenet_g3_w3d4"
    shufflenet_g1_wd2 = "shufflenet_g1_wd2"
    shufflenet_g3_wd2 = "shufflenet_g3_wd2"
    shufflenet_g1_wd4 = "shufflenet_g1_wd4"
    shufflenet_g3_wd4 = "shufflenet_g3_wd4"
    shufflenetv2_wd2 = "shufflenetv2_wd2"
    shufflenetv2_w1 = "shufflenetv2_w1"
    shufflenetv2_w3d2 = "shufflenetv2_w3d2"
    shufflenetv2_w2 = "shufflenetv2_w2"
    shufflenetv2b_wd2 = "shufflenetv2b_wd2"
    shufflenetv2b_w1 = "shufflenetv2b_w1"
    shufflenetv2b_w3d2 = "shufflenetv2b_w3d2"
    shufflenetv2b_w2 = "shufflenetv2b_w2"
    menet108_8x1_g3 = "menet108_8x1_g3"
    menet128_8x1_g4 = "menet128_8x1_g4"
    menet160_8x1_g8 = "menet160_8x1_g8"
    menet228_12x1_g3 = "menet228_12x1_g3"
    menet256_12x1_g4 = "menet256_12x1_g4"
    menet348_12x1_g3 = "menet348_12x1_g3"
    menet352_12x1_g8 = "menet352_12x1_g8"
    menet456_24x1_g3 = "menet456_24x1_g3"
    mobilenet_w1 = "mobilenet_w1"
    mobilenet_w3d4 = "mobilenet_w3d4"
    mobilenet_wd2 = "mobilenet_wd2"
    mobilenet_wd4 = "mobilenet_wd4"
    fdmobilenet_w1 = "fdmobilenet_w1"
    fdmobilenet_w3d4 = "fdmobilenet_w3d4"
    fdmobilenet_wd2 = "fdmobilenet_wd2"
    fdmobilenet_wd4 = "fdmobilenet_wd4"
    mobilenetv2_w1 = "mobilenetv2_w1"
    mobilenetv2_w3d4 = "mobilenetv2_w3d4"
    mobilenetv2_wd2 = "mobilenetv2_wd2"
    mobilenetv2_wd4 = "mobilenetv2_wd4"
    mobilenetv2b_w1 = "mobilenetv2b_w1"
    mobilenetv2b_w3d4 = "mobilenetv2b_w3d4"
    mobilenetv2b_wd2 = "mobilenetv2b_wd2"
    mobilenetv2b_wd4 = "mobilenetv2b_wd4"
    mobilenetv3_small_w7d20 = "mobilenetv3_small_w7d20"
    mobilenetv3_small_wd2 = "mobilenetv3_small_wd2"
    mobilenetv3_small_w3d4 = "mobilenetv3_small_w3d4"
    mobilenetv3_small_w1 = "mobilenetv3_small_w1"
    mobilenetv3_small_w5d4 = "mobilenetv3_small_w5d4"
    mobilenetv3_large_w7d20 = "mobilenetv3_large_w7d20"
    mobilenetv3_large_wd2 = "mobilenetv3_large_wd2"
    mobilenetv3_large_w3d4 = "mobilenetv3_large_w3d4"
    mobilenetv3_large_w1 = "mobilenetv3_large_w1"
    mobilenetv3_large_w5d4 = "mobilenetv3_large_w5d4"
    igcv3_w1 = "igcv3_w1"
    igcv3_w3d4 = "igcv3_w3d4"
    igcv3_wd2 = "igcv3_wd2"
    igcv3_wd4 = "igcv3_wd4"
    ghostnet = "ghostnet"
    mnasnet_b1 = "mnasnet_b1"
    mnasnet_a1 = "mnasnet_a1"
    mnasnet_small = "mnasnet_small"
    darts = "darts"
    proxylessnas_cpu = "proxylessnas_cpu"
    proxylessnas_gpu = "proxylessnas_gpu"
    proxylessnas_mobile = "proxylessnas_mobile"
    proxylessnas_mobile14 = "proxylessnas_mobile14"
    fbnet_cb = "fbnet_cb"
    xception = "xception"
    inceptionv3 = "inceptionv3"
    inceptionv4 = "inceptionv4"
    inceptionresnetv2 = "inceptionresnetv2"
    polynet = "polynet"
    nasnet_4a1056 = "nasnet_4a1056"
    nasnet_6a4032 = "nasnet_6a4032"
    pnasnet5large = "pnasnet5large"
    spnasnet = "spnasnet"
    efficientnet_b0 = "efficientnet_b0"
    efficientnet_b1 = "efficientnet_b1"
    efficientnet_b2 = "efficientnet_b2"
    efficientnet_b3 = "efficientnet_b3"
    efficientnet_b4 = "efficientnet_b4"
    efficientnet_b5 = "efficientnet_b5"
    efficientnet_b6 = "efficientnet_b6"
    efficientnet_b7 = "efficientnet_b7"
    efficientnet_b8 = "efficientnet_b8"
    efficientnet_b0b = "efficientnet_b0b"
    efficientnet_b1b = "efficientnet_b1b"
    efficientnet_b2b = "efficientnet_b2b"
    efficientnet_b3b = "efficientnet_b3b"
    efficientnet_b4b = "efficientnet_b4b"
    efficientnet_b5b = "efficientnet_b5b"
    efficientnet_b6b = "efficientnet_b6b"
    efficientnet_b7b = "efficientnet_b7b"
    efficientnet_b0c = "efficientnet_b0c"
    efficientnet_b1c = "efficientnet_b1c"
    efficientnet_b2c = "efficientnet_b2c"
    efficientnet_b3c = "efficientnet_b3c"
    efficientnet_b4c = "efficientnet_b4c"
    efficientnet_b5c = "efficientnet_b5c"
    efficientnet_b6c = "efficientnet_b6c"
    efficientnet_b7c = "efficientnet_b7c"
    efficientnet_b8c = "efficientnet_b8c"
    efficientnet_edge_small_b = "efficientnet_edge_small_b"
    efficientnet_edge_medium_b = "efficientnet_edge_medium_b"
    efficientnet_edge_large_b = "efficientnet_edge_large_b"
    mixnet_s = "mixnet_s"
    mixnet_m = "mixnet_m"
    mixnet_l = "mixnet_l"
    nin_cifar10 = "nin_cifar10"
    nin_cifar100 = "nin_cifar100"
    nin_svhn = "nin_svhn"
    resnet20_cifar10 = "resnet20_cifar10"
    resnet20_cifar100 = "resnet20_cifar100"
    resnet20_svhn = "resnet20_svhn"
    resnet56_cifar10 = "resnet56_cifar10"
    resnet56_cifar100 = "resnet56_cifar100"
    resnet56_svhn = "resnet56_svhn"
    resnet110_cifar10 = "resnet110_cifar10"
    resnet110_cifar100 = "resnet110_cifar100"
    resnet110_svhn = "resnet110_svhn"
    resnet164bn_cifar10 = "resnet164bn_cifar10"
    resnet164bn_cifar100 = "resnet164bn_cifar100"
    resnet164bn_svhn = "resnet164bn_svhn"
    resnet272bn_cifar10 = "resnet272bn_cifar10"
    resnet272bn_cifar100 = "resnet272bn_cifar100"
    resnet272bn_svhn = "resnet272bn_svhn"
    resnet542bn_cifar10 = "resnet542bn_cifar10"
    resnet542bn_cifar100 = "resnet542bn_cifar100"
    resnet542bn_svhn = "resnet542bn_svhn"
    resnet1001_cifar10 = "resnet1001_cifar10"
    resnet1001_cifar100 = "resnet1001_cifar100"
    resnet1001_svhn = "resnet1001_svhn"
    resnet1202_cifar10 = "resnet1202_cifar10"
    resnet1202_cifar100 = "resnet1202_cifar100"
    resnet1202_svhn = "resnet1202_svhn"
    preresnet20_cifar10 = "preresnet20_cifar10"
    preresnet20_cifar100 = "preresnet20_cifar100"
    preresnet20_svhn = "preresnet20_svhn"
    preresnet56_cifar10 = "preresnet56_cifar10"
    preresnet56_cifar100 = "preresnet56_cifar100"
    preresnet56_svhn = "preresnet56_svhn"
    preresnet110_cifar10 = "preresnet110_cifar10"
    preresnet110_cifar100 = "preresnet110_cifar100"
    preresnet110_svhn = "preresnet110_svhn"
    preresnet164bn_cifar10 = "preresnet164bn_cifar10"
    preresnet164bn_cifar100 = "preresnet164bn_cifar100"
    preresnet164bn_svhn = "preresnet164bn_svhn"
    preresnet272bn_cifar10 = "preresnet272bn_cifar10"
    preresnet272bn_cifar100 = "preresnet272bn_cifar100"
    preresnet272bn_svhn = "preresnet272bn_svhn"
    preresnet542bn_cifar10 = "preresnet542bn_cifar10"
    preresnet542bn_cifar100 = "preresnet542bn_cifar100"
    preresnet542bn_svhn = "preresnet542bn_svhn"
    preresnet1001_cifar10 = "preresnet1001_cifar10"
    preresnet1001_cifar100 = "preresnet1001_cifar100"
    preresnet1001_svhn = "preresnet1001_svhn"
    preresnet1202_cifar10 = "preresnet1202_cifar10"
    preresnet1202_cifar100 = "preresnet1202_cifar100"
    preresnet1202_svhn = "preresnet1202_svhn"
    resnext20_16x4d_cifar10 = "resnext20_16x4d_cifar10"
    resnext20_16x4d_cifar100 = "resnext20_16x4d_cifar100"
    resnext20_16x4d_svhn = "resnext20_16x4d_svhn"
    resnext20_32x2d_cifar10 = "resnext20_32x2d_cifar10"
    resnext20_32x2d_cifar100 = "resnext20_32x2d_cifar100"
    resnext20_32x2d_svhn = "resnext20_32x2d_svhn"
    resnext20_32x4d_cifar10 = "resnext20_32x4d_cifar10"
    resnext20_32x4d_cifar100 = "resnext20_32x4d_cifar100"
    resnext20_32x4d_svhn = "resnext20_32x4d_svhn"
    resnext29_32x4d_cifar10 = "resnext29_32x4d_cifar10"
    resnext29_32x4d_cifar100 = "resnext29_32x4d_cifar100"
    resnext29_32x4d_svhn = "resnext29_32x4d_svhn"
    resnext29_16x64d_cifar10 = "resnext29_16x64d_cifar10"
    resnext29_16x64d_cifar100 = "resnext29_16x64d_cifar100"
    resnext29_16x64d_svhn = "resnext29_16x64d_svhn"
    resnext272_1x64d_cifar10 = "resnext272_1x64d_cifar10"
    resnext272_1x64d_cifar100 = "resnext272_1x64d_cifar100"
    resnext272_1x64d_svhn = "resnext272_1x64d_svhn"
    resnext272_2x32d_cifar10 = "resnext272_2x32d_cifar10"
    resnext272_2x32d_cifar100 = "resnext272_2x32d_cifar100"
    resnext272_2x32d_svhn = "resnext272_2x32d_svhn"
    seresnet20_cifar10 = "seresnet20_cifar10"
    seresnet20_cifar100 = "seresnet20_cifar100"
    seresnet20_svhn = "seresnet20_svhn"
    seresnet56_cifar10 = "seresnet56_cifar10"
    seresnet56_cifar100 = "seresnet56_cifar100"
    seresnet56_svhn = "seresnet56_svhn"
    seresnet110_cifar10 = "seresnet110_cifar10"
    seresnet110_cifar100 = "seresnet110_cifar100"
    seresnet110_svhn = "seresnet110_svhn"
    seresnet164bn_cifar10 = "seresnet164bn_cifar10"
    seresnet164bn_cifar100 = "seresnet164bn_cifar100"
    seresnet164bn_svhn = "seresnet164bn_svhn"
    seresnet272bn_cifar10 = "seresnet272bn_cifar10"
    seresnet272bn_cifar100 = "seresnet272bn_cifar100"
    seresnet272bn_svhn = "seresnet272bn_svhn"
    seresnet542bn_cifar10 = "seresnet542bn_cifar10"
    seresnet542bn_cifar100 = "seresnet542bn_cifar100"
    seresnet542bn_svhn = "seresnet542bn_svhn"
    seresnet1001_cifar10 = "seresnet1001_cifar10"
    seresnet1001_cifar100 = "seresnet1001_cifar100"
    seresnet1001_svhn = "seresnet1001_svhn"
    seresnet1202_cifar10 = "seresnet1202_cifar10"
    seresnet1202_cifar100 = "seresnet1202_cifar100"
    seresnet1202_svhn = "seresnet1202_svhn"
    sepreresnet20_cifar10 = "sepreresnet20_cifar10"
    sepreresnet20_cifar100 = "sepreresnet20_cifar100"
    sepreresnet20_svhn = "sepreresnet20_svhn"
    sepreresnet56_cifar10 = "sepreresnet56_cifar10"
    sepreresnet56_cifar100 = "sepreresnet56_cifar100"
    sepreresnet56_svhn = "sepreresnet56_svhn"
    sepreresnet110_cifar10 = "sepreresnet110_cifar10"
    sepreresnet110_cifar100 = "sepreresnet110_cifar100"
    sepreresnet110_svhn = "sepreresnet110_svhn"
    sepreresnet164bn_cifar10 = "sepreresnet164bn_cifar10"
    sepreresnet164bn_cifar100 = "sepreresnet164bn_cifar100"
    sepreresnet164bn_svhn = "sepreresnet164bn_svhn"
    sepreresnet272bn_cifar10 = "sepreresnet272bn_cifar10"
    sepreresnet272bn_cifar100 = "sepreresnet272bn_cifar100"
    sepreresnet272bn_svhn = "sepreresnet272bn_svhn"
    sepreresnet542bn_cifar10 = "sepreresnet542bn_cifar10"
    sepreresnet542bn_cifar100 = "sepreresnet542bn_cifar100"
    sepreresnet542bn_svhn = "sepreresnet542bn_svhn"
    sepreresnet1001_cifar10 = "sepreresnet1001_cifar10"
    sepreresnet1001_cifar100 = "sepreresnet1001_cifar100"
    sepreresnet1001_svhn = "sepreresnet1001_svhn"
    sepreresnet1202_cifar10 = "sepreresnet1202_cifar10"
    sepreresnet1202_cifar100 = "sepreresnet1202_cifar100"
    sepreresnet1202_svhn = "sepreresnet1202_svhn"
    pyramidnet110_a48_cifar10 = "pyramidnet110_a48_cifar10"
    pyramidnet110_a48_cifar100 = "pyramidnet110_a48_cifar100"
    pyramidnet110_a48_svhn = "pyramidnet110_a48_svhn"
    pyramidnet110_a84_cifar10 = "pyramidnet110_a84_cifar10"
    pyramidnet110_a84_cifar100 = "pyramidnet110_a84_cifar100"
    pyramidnet110_a84_svhn = "pyramidnet110_a84_svhn"
    pyramidnet110_a270_cifar10 = "pyramidnet110_a270_cifar10"
    pyramidnet110_a270_cifar100 = "pyramidnet110_a270_cifar100"
    pyramidnet110_a270_svhn = "pyramidnet110_a270_svhn"
    pyramidnet164_a270_bn_cifar10 = "pyramidnet164_a270_bn_cifar10"
    pyramidnet164_a270_bn_cifar100 = "pyramidnet164_a270_bn_cifar100"
    pyramidnet164_a270_bn_svhn = "pyramidnet164_a270_bn_svhn"
    pyramidnet200_a240_bn_cifar10 = "pyramidnet200_a240_bn_cifar10"
    pyramidnet200_a240_bn_cifar100 = "pyramidnet200_a240_bn_cifar100"
    pyramidnet200_a240_bn_svhn = "pyramidnet200_a240_bn_svhn"
    pyramidnet236_a220_bn_cifar10 = "pyramidnet236_a220_bn_cifar10"
    pyramidnet236_a220_bn_cifar100 = "pyramidnet236_a220_bn_cifar100"
    pyramidnet236_a220_bn_svhn = "pyramidnet236_a220_bn_svhn"
    pyramidnet272_a200_bn_cifar10 = "pyramidnet272_a200_bn_cifar10"
    pyramidnet272_a200_bn_cifar100 = "pyramidnet272_a200_bn_cifar100"
    pyramidnet272_a200_bn_svhn = "pyramidnet272_a200_bn_svhn"
    densenet40_k12_cifar10 = "densenet40_k12_cifar10"
    densenet40_k12_cifar100 = "densenet40_k12_cifar100"
    densenet40_k12_svhn = "densenet40_k12_svhn"
    densenet40_k12_bc_cifar10 = "densenet40_k12_bc_cifar10"
    densenet40_k12_bc_cifar100 = "densenet40_k12_bc_cifar100"
    densenet40_k12_bc_svhn = "densenet40_k12_bc_svhn"
    densenet40_k24_bc_cifar10 = "densenet40_k24_bc_cifar10"
    densenet40_k24_bc_cifar100 = "densenet40_k24_bc_cifar100"
    densenet40_k24_bc_svhn = "densenet40_k24_bc_svhn"
    densenet40_k36_bc_cifar10 = "densenet40_k36_bc_cifar10"
    densenet40_k36_bc_cifar100 = "densenet40_k36_bc_cifar100"
    densenet40_k36_bc_svhn = "densenet40_k36_bc_svhn"
    densenet100_k12_cifar10 = "densenet100_k12_cifar10"
    densenet100_k12_cifar100 = "densenet100_k12_cifar100"
    densenet100_k12_svhn = "densenet100_k12_svhn"
    densenet100_k24_cifar10 = "densenet100_k24_cifar10"
    densenet100_k24_cifar100 = "densenet100_k24_cifar100"
    densenet100_k24_svhn = "densenet100_k24_svhn"
    densenet100_k12_bc_cifar10 = "densenet100_k12_bc_cifar10"
    densenet100_k12_bc_cifar100 = "densenet100_k12_bc_cifar100"
    densenet100_k12_bc_svhn = "densenet100_k12_bc_svhn"
    densenet190_k40_bc_cifar10 = "densenet190_k40_bc_cifar10"
    densenet190_k40_bc_cifar100 = "densenet190_k40_bc_cifar100"
    densenet190_k40_bc_svhn = "densenet190_k40_bc_svhn"
    densenet250_k24_bc_cifar10 = "densenet250_k24_bc_cifar10"
    densenet250_k24_bc_cifar100 = "densenet250_k24_bc_cifar100"
    densenet250_k24_bc_svhn = "densenet250_k24_bc_svhn"
    xdensenet40_2_k24_bc_cifar10 = "xdensenet40_2_k24_bc_cifar10"
    xdensenet40_2_k24_bc_cifar100 = "xdensenet40_2_k24_bc_cifar100"
    xdensenet40_2_k24_bc_svhn = "xdensenet40_2_k24_bc_svhn"
    xdensenet40_2_k36_bc_cifar10 = "xdensenet40_2_k36_bc_cifar10"
    xdensenet40_2_k36_bc_cifar100 = "xdensenet40_2_k36_bc_cifar100"
    xdensenet40_2_k36_bc_svhn = "xdensenet40_2_k36_bc_svhn"
    wrn16_10_cifar10 = "wrn16_10_cifar10"
    wrn16_10_cifar100 = "wrn16_10_cifar100"
    wrn16_10_svhn = "wrn16_10_svhn"
    wrn28_10_cifar10 = "wrn28_10_cifar10"
    wrn28_10_cifar100 = "wrn28_10_cifar100"
    wrn28_10_svhn = "wrn28_10_svhn"
    wrn40_8_cifar10 = "wrn40_8_cifar10"
    wrn40_8_cifar100 = "wrn40_8_cifar100"
    wrn40_8_svhn = "wrn40_8_svhn"
    wrn20_10_1bit_cifar10 = "wrn20_10_1bit_cifar10"
    wrn20_10_1bit_cifar100 = "wrn20_10_1bit_cifar100"
    wrn20_10_1bit_svhn = "wrn20_10_1bit_svhn"
    wrn20_10_32bit_cifar10 = "wrn20_10_32bit_cifar10"
    wrn20_10_32bit_cifar100 = "wrn20_10_32bit_cifar100"
    wrn20_10_32bit_svhn = "wrn20_10_32bit_svhn"
    ror3_56_cifar10 = "ror3_56_cifar10"
    ror3_56_cifar100 = "ror3_56_cifar100"
    ror3_56_svhn = "ror3_56_svhn"
    ror3_110_cifar10 = "ror3_110_cifar10"
    ror3_110_cifar100 = "ror3_110_cifar100"
    ror3_110_svhn = "ror3_110_svhn"
    ror3_164_cifar10 = "ror3_164_cifar10"
    ror3_164_cifar100 = "ror3_164_cifar100"
    ror3_164_svhn = "ror3_164_svhn"
    rir_cifar10 = "rir_cifar10"
    rir_cifar100 = "rir_cifar100"
    rir_svhn = "rir_svhn"
    msdnet22_cifar10 = "msdnet22_cifar10"
    resdropresnet20_cifar10 = "resdropresnet20_cifar10"
    resdropresnet20_cifar100 = "resdropresnet20_cifar100"
    resdropresnet20_svhn = "resdropresnet20_svhn"
    shakeshakeresnet20_2x16d_cifar10 = "shakeshakeresnet20_2x16d_cifar10"
    shakeshakeresnet20_2x16d_cifar100 = "shakeshakeresnet20_2x16d_cifar100"
    shakeshakeresnet20_2x16d_svhn = "shakeshakeresnet20_2x16d_svhn"
    shakeshakeresnet26_2x32d_cifar10 = "shakeshakeresnet26_2x32d_cifar10"
    shakeshakeresnet26_2x32d_cifar100 = "shakeshakeresnet26_2x32d_cifar100"
    shakeshakeresnet26_2x32d_svhn = "shakeshakeresnet26_2x32d_svhn"
    shakedropresnet20_cifar10 = "shakedropresnet20_cifar10"
    shakedropresnet20_cifar100 = "shakedropresnet20_cifar100"
    shakedropresnet20_svhn = "shakedropresnet20_svhn"
    fractalnet_cifar10 = "fractalnet_cifar10"
    fractalnet_cifar100 = "fractalnet_cifar100"
    diaresnet20_cifar10 = "diaresnet20_cifar10"
    diaresnet20_cifar100 = "diaresnet20_cifar100"
    diaresnet20_svhn = "diaresnet20_svhn"
    diaresnet56_cifar10 = "diaresnet56_cifar10"
    diaresnet56_cifar100 = "diaresnet56_cifar100"
    diaresnet56_svhn = "diaresnet56_svhn"
    diaresnet110_cifar10 = "diaresnet110_cifar10"
    diaresnet110_cifar100 = "diaresnet110_cifar100"
    diaresnet110_svhn = "diaresnet110_svhn"
    diaresnet164bn_cifar10 = "diaresnet164bn_cifar10"
    diaresnet164bn_cifar100 = "diaresnet164bn_cifar100"
    diaresnet164bn_svhn = "diaresnet164bn_svhn"
    diaresnet1001_cifar10 = "diaresnet1001_cifar10"
    diaresnet1001_cifar100 = "diaresnet1001_cifar100"
    diaresnet1001_svhn = "diaresnet1001_svhn"
    diaresnet1202_cifar10 = "diaresnet1202_cifar10"
    diaresnet1202_cifar100 = "diaresnet1202_cifar100"
    diaresnet1202_svhn = "diaresnet1202_svhn"
    diapreresnet20_cifar10 = "diapreresnet20_cifar10"
    diapreresnet20_cifar100 = "diapreresnet20_cifar100"
    diapreresnet20_svhn = "diapreresnet20_svhn"
    diapreresnet56_cifar10 = "diapreresnet56_cifar10"
    diapreresnet56_cifar100 = "diapreresnet56_cifar100"
    diapreresnet56_svhn = "diapreresnet56_svhn"
    diapreresnet110_cifar10 = "diapreresnet110_cifar10"
    diapreresnet110_cifar100 = "diapreresnet110_cifar100"
    diapreresnet110_svhn = "diapreresnet110_svhn"
    diapreresnet164bn_cifar10 = "diapreresnet164bn_cifar10"
    diapreresnet164bn_cifar100 = "diapreresnet164bn_cifar100"
    diapreresnet164bn_svhn = "diapreresnet164bn_svhn"
    diapreresnet1001_cifar10 = "diapreresnet1001_cifar10"
    diapreresnet1001_cifar100 = "diapreresnet1001_cifar100"
    diapreresnet1001_svhn = "diapreresnet1001_svhn"
    diapreresnet1202_cifar10 = "diapreresnet1202_cifar10"
    diapreresnet1202_cifar100 = "diapreresnet1202_cifar100"
    diapreresnet1202_svhn = "diapreresnet1202_svhn"
    isqrtcovresnet18 = "isqrtcovresnet18"
    isqrtcovresnet34 = "isqrtcovresnet34"
    isqrtcovresnet50 = "isqrtcovresnet50"
    isqrtcovresnet50b = "isqrtcovresnet50b"
    isqrtcovresnet101 = "isqrtcovresnet101"
    isqrtcovresnet101b = "isqrtcovresnet101b"
    resneta50b = "resneta50b"
    resneta101b = "resneta101b"
    resneta152b = "resneta152b"
    resnetd50b = "resnetd50b"
    resnetd101b = "resnetd101b"
    resnetd152b = "resnetd152b"
    fastseresnet101b = "fastseresnet101b"
    octresnet10_ad2 = "octresnet10_ad2"
    octresnet50b_ad2 = "octresnet50b_ad2"
    resnet10_cub = "resnet10_cub"
    resnet12_cub = "resnet12_cub"
    resnet14_cub = "resnet14_cub"
    resnetbc14b_cub = "resnetbc14b_cub"
    resnet16_cub = "resnet16_cub"
    resnet18_cub = "resnet18_cub"
    resnet26_cub = "resnet26_cub"
    resnetbc26b_cub = "resnetbc26b_cub"
    resnet34_cub = "resnet34_cub"
    resnetbc38b_cub = "resnetbc38b_cub"
    resnet50_cub = "resnet50_cub"
    resnet50b_cub = "resnet50b_cub"
    resnet101_cub = "resnet101_cub"
    resnet101b_cub = "resnet101b_cub"
    resnet152_cub = "resnet152_cub"
    resnet152b_cub = "resnet152b_cub"
    resnet200_cub = "resnet200_cub"
    resnet200b_cub = "resnet200b_cub"
    seresnet10_cub = "seresnet10_cub"
    seresnet12_cub = "seresnet12_cub"
    seresnet14_cub = "seresnet14_cub"
    seresnetbc14b_cub = "seresnetbc14b_cub"
    seresnet16_cub = "seresnet16_cub"
    seresnet18_cub = "seresnet18_cub"
    seresnet26_cub = "seresnet26_cub"
    seresnetbc26b_cub = "seresnetbc26b_cub"
    seresnet34_cub = "seresnet34_cub"
    seresnetbc38b_cub = "seresnetbc38b_cub"
    seresnet50_cub = "seresnet50_cub"
    seresnet50b_cub = "seresnet50b_cub"
    seresnet101_cub = "seresnet101_cub"
    seresnet101b_cub = "seresnet101b_cub"
    seresnet152_cub = "seresnet152_cub"
    seresnet152b_cub = "seresnet152b_cub"
    seresnet200_cub = "seresnet200_cub"
    seresnet200b_cub = "seresnet200b_cub"
    mobilenet_w1_cub = "mobilenet_w1_cub"
    mobilenet_w3d4_cub = "mobilenet_w3d4_cub"
    mobilenet_wd2_cub = "mobilenet_wd2_cub"
    mobilenet_wd4_cub = "mobilenet_wd4_cub"
    fdmobilenet_w1_cub = "fdmobilenet_w1_cub"
    fdmobilenet_w3d4_cub = "fdmobilenet_w3d4_cub"
    fdmobilenet_wd2_cub = "fdmobilenet_wd2_cub"
    fdmobilenet_wd4_cub = "fdmobilenet_wd4_cub"
    proxylessnas_cpu_cub = "proxylessnas_cpu_cub"
    proxylessnas_gpu_cub = "proxylessnas_gpu_cub"
    proxylessnas_mobile_cub = "proxylessnas_mobile_cub"
    proxylessnas_mobile14_cub = "proxylessnas_mobile14_cub"
    ntsnet_cub = "ntsnet_cub"
    fcn8sd_resnetd50b_voc = "fcn8sd_resnetd50b_voc"
    fcn8sd_resnetd101b_voc = "fcn8sd_resnetd101b_voc"
    fcn8sd_resnetd50b_coco = "fcn8sd_resnetd50b_coco"
    fcn8sd_resnetd101b_coco = "fcn8sd_resnetd101b_coco"
    fcn8sd_resnetd50b_ade20k = "fcn8sd_resnetd50b_ade20k"
    fcn8sd_resnetd101b_ade20k = "fcn8sd_resnetd101b_ade20k"
    fcn8sd_resnetd50b_cityscapes = "fcn8sd_resnetd50b_cityscapes"
    fcn8sd_resnetd101b_cityscapes = "fcn8sd_resnetd101b_cityscapes"
    pspnet_resnetd50b_voc = "pspnet_resnetd50b_voc"
    pspnet_resnetd101b_voc = "pspnet_resnetd101b_voc"
    pspnet_resnetd50b_coco = "pspnet_resnetd50b_coco"
    pspnet_resnetd101b_coco = "pspnet_resnetd101b_coco"
    pspnet_resnetd50b_ade20k = "pspnet_resnetd50b_ade20k"
    pspnet_resnetd101b_ade20k = "pspnet_resnetd101b_ade20k"
    pspnet_resnetd50b_cityscapes = "pspnet_resnetd50b_cityscapes"
    pspnet_resnetd101b_cityscapes = "pspnet_resnetd101b_cityscapes"
    deeplabv3_resnetd50b_voc = "deeplabv3_resnetd50b_voc"
    deeplabv3_resnetd101b_voc = "deeplabv3_resnetd101b_voc"
    deeplabv3_resnetd152b_voc = "deeplabv3_resnetd152b_voc"
    deeplabv3_resnetd50b_coco = "deeplabv3_resnetd50b_coco"
    deeplabv3_resnetd101b_coco = "deeplabv3_resnetd101b_coco"
    deeplabv3_resnetd152b_coco = "deeplabv3_resnetd152b_coco"
    deeplabv3_resnetd50b_ade20k = "deeplabv3_resnetd50b_ade20k"
    deeplabv3_resnetd101b_ade20k = "deeplabv3_resnetd101b_ade20k"
    deeplabv3_resnetd50b_cityscapes = "deeplabv3_resnetd50b_cityscapes"
    deeplabv3_resnetd101b_cityscapes = "deeplabv3_resnetd101b_cityscapes"
    icnet_resnetd50b_cityscapes = "icnet_resnetd50b_cityscapes"
    sinet_cityscapes = "sinet_cityscapes"
    alphapose_fastseresnet101b_coco = "alphapose_fastseresnet101b_coco"
    simplepose_resnet18_coco = "simplepose_resnet18_coco"
    simplepose_resnet50b_coco = "simplepose_resnet50b_coco"
    simplepose_resnet101b_coco = "simplepose_resnet101b_coco"
    simplepose_resnet152b_coco = "simplepose_resnet152b_coco"
    simplepose_resneta50b_coco = "simplepose_resneta50b_coco"
    simplepose_resneta101b_coco = "simplepose_resneta101b_coco"
    simplepose_resneta152b_coco = "simplepose_resneta152b_coco"
    simplepose_mobile_resnet18_coco = "simplepose_mobile_resnet18_coco"
    simplepose_mobile_resnet50b_coco = "simplepose_mobile_resnet50b_coco"
    simplepose_mobile_mobilenet_w1_coco = "simplepose_mobile_mobilenet_w1_coco"
    simplepose_mobile_mobilenetv2b_w1_coco = "simplepose_mobile_mobilenetv2b_w1_coco"
    simplepose_mobile_mobilenetv3_small_w1_coco = "simplepose_mobile_mobilenetv3_small_w1_coco"
    simplepose_mobile_mobilenetv3_large_w1_coco = "simplepose_mobile_mobilenetv3_large_w1_coco"
    lwopenpose2d_mobilenet_cmupan_coco = "lwopenpose2d_mobilenet_cmupan_coco"
    lwopenpose3d_mobilenet_cmupan_coco = "lwopenpose3d_mobilenet_cmupan_coco"
    ibppose_coco = "ibppose_coco"
    prnet = "prnet"
    centernet_resnet18_voc = "centernet_resnet18_voc"
    centernet_resnet18_coco = "centernet_resnet18_coco"
    centernet_resnet50b_voc = "centernet_resnet50b_voc"
    centernet_resnet50b_coco = "centernet_resnet50b_coco"
    centernet_resnet101b_voc = "centernet_resnet101b_voc"
    centernet_resnet101b_coco = "centernet_resnet101b_coco"
    lffd20x5s320v2_widerface = "lffd20x5s320v2_widerface"
    lffd25x8s560v1_widerface = "lffd25x8s560v1_widerface"
    voca8flame = "voca8flame"
    superpointnet = "superpointnet"
    resnext101_32x8d = "resnext101_32x8d"
    wide_resnet101_2 = "wide_resnet101_2"
    wide_resnet50_2 = "wide_resnet50_2"
    googlenet = "googlenet"
    mnasnet0_5 = "mnasnet0_5"
    mnasnet0_75 = "mnasnet0_75"
    mnasnet1_0 = "mnasnet1_0"
    mnasnet1_3 = "mnasnet1_3"
    mobilenet_v2 = "mobilenet_v2"
    inception_v3 = "inception_v3"
    shufflenet_v2_x0_5 = "shufflenet_v2_x0_5"
    shufflenet_v2_x1_0 = "shufflenet_v2_x1_0"
    shufflenet_v2_x1_5 = "shufflenet_v2_x1_5"
    shufflenet_v2_x2_0 = "shufflenet_v2_x2_0"



from wcf import train, TrainValConfigBase, val,t,EasyTransform,models_names


class Config(TrainValConfigBase):
    MODEL_TYPE = models_names.shufflenet_v2_x0_5
    # MODEL_TYPE = models_names.resnet18
    TAG = '[%s]'%(MODEL_TYPE)
    GEN_CLASSES_FILE = True
    USE_tqdm_TRAIN = True
    INPUT_SIZE = (252,196)
    BATCH_SIZE = 64
    MAX_EPOCHS = 200
    BALANCE_CLASSES = True
    VAL_INTERVAL = 1
    WEIGHTS_SAVE_INTERVAL = 1
    WEIGHTS_INIT = 'weights/training/model_best.pkl'
    # WEIGHTS_INIT = '/home/ars/sda6/work/play/wk-classify/local/cigarette_defect/weights/trained/cigarette_box_defect/model_final_[acc=1].pkl'
    TRAIN_DIR = '/home/ars/sda5/data/projects/烟盒/data/现场采集好坏烟照片/相机1-train'
    VAL_DIR = '/home/ars/sda5/data/projects/烟盒/data/现场采集好坏烟照片/相机1-val'
    # VAL_DIR = '/home/ars/sda5/data/projects/烟盒/data/现场采集好坏烟照片/相机1'
    # VAL_DIR = '/home/ars/sda5/data/烟盒/datasets/dataset/val'
    val_transform = EasyTransform([
        t.Resize(INPUT_SIZE[::-1]),
        t.SaveToDir('data/test'),
        t.ToTensor(),
    ])
    train_transform = EasyTransform([
        t.ColorJitter(brightness=0.2, contrast=0, saturation=0, hue=0),
        # t.ColorJitter(brightness=0.2, contrast=0.1, saturation=0.1, hue=0.05),
        # t.RandomHorizontalFlip(),
        # t.RandomVerticalFlip(),
        # t.RandomRotate(360),
        t.RandomTranslate(30),
        t.RandomBlur(p=0.3, radius=1),
        t.RandomSPNoise(p=0.3),
        *val_transform,
    ])
    # def get_model(self, num_classes=None):
        # return TwoLayerNet(17)
        # return LightNet(17)



if __name__ == '__main__':
    cfg = Config()
    train(cfg)
    # res=val(cfg)
    # print(res)



from .ptcv_models import ptcv_models_dict
from torch import nn
from .torchvision_models import torchvision_models_dict



def load_model(model_type,num_classes,pretrained=False):
    if callable(model_type):
        return model_type(pretrained=pretrained,num_classes=num_classes)
    if model_type in torchvision_models_dict.keys():
        model=torchvision_models_dict[model_type](pretrained=pretrained)
        model.fc = nn.Linear(model.fc.in_features, num_classes)
    elif model_type in ptcv_models_dict.keys():
        model=ptcv_models_dict[model_type](pretrained=pretrained)
        model.features.final_pool=nn.AdaptiveAvgPool2d((1, 1))
        if isinstance(model.output,nn.Linear):
            model.output=nn.Linear(model.output.in_features, num_classes)
        else:
            model.output.fc = nn.Linear(model.output.fc.in_features, num_classes)
    else:
        raise Exception('Model %s not found.'%(model_type))
    return model




import os,shutil,glob

def encrypt_model(src,dst=None,key=0xef):
    dst=dst or src+'.encrypted'
    with open(src ,'rb') as f:
        data=f.read()
    data2=bytearray(data)
    for i,B in enumerate(data):
        data2[i]=B^key
    data2=bytes(data2)
    with open(dst,'wb') as f:
        f.write(data2)
def decrypt_model(src,dst=None,key=0xef):
    dst=dst or src+'.decrypted'
    with open(src ,'rb') as f:
        data=f.read()
    data2=bytearray(data)
    for i,B in enumerate(data):
        data2[i]=B^key
    data2=bytes(data2)
    with open(dst,'wb') as f:
        f.write(data2)

if __name__ == '__main__':
    pass
    encrypt_model(
        src='/home/ars/sda6/work/play/wk-classify/local/capacitor_color1_face2/model_best_[capacitor][0916][resnet18][face1][epoch=5&acc=0.9350].onnx'
    )